# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""
PyTest suite for testing if each run.py is aligned with module specification:

> Status: this code relates to the _recipe_ and is a _proposition_
"""
import pytest
import os

from shrike.pipeline.testing.module_run_tests import module_run_get_arg_parser
from shrike.pipeline.testing.module_run_tests import (
    if_arguments_from_module_spec_match_script_argparse,
)
from shrike.pipeline.testing.module_run_tests import run_py_import
from shrike.pipeline.testing.module_run_tests import (
    module_spec_yaml_exists_and_is_parsable,
)

# from shrike.pipeline.testing.module_run_tests import (
#    script_main_with_synthetic_arguments,
# )
from shrike.pipeline.testing.module_run_tests import module_uses_private_acr
from shrike.pipeline.testing.module_run_tests import (
    module_uses_private_python_feed,
)

# modules that should ALSO pass advanced tests (design pattern)
MODULE_STEP_FOLDERS = [
    os.path.abspath(
        os.path.join(os.path.dirname(__file__), "sample", "steps", "stats_passthrough")
    ),
    os.path.abspath(
        os.path.join(
            os.path.dirname(__file__),
            "sample",
            "steps",
            "stats_passthrough_windows",
        )
    ),
    os.path.abspath(
        os.path.join(os.path.dirname(__file__), "sample", "steps", "multinode_trainer")
    ),
    os.path.abspath(
        os.path.join(os.path.dirname(__file__), "sample", "steps", "spark_hello")
    ),
    # os.path.abspath(
    #     os.path.join(os.path.dirname(__file__), "sample", "steps", "convert_tsv_to_ss")
    # )
]

### BASIC TESTS ###
# for basic module designs (minimal wrappers)


@pytest.mark.parametrize("module", MODULE_STEP_FOLDERS)
def test_run_py_import(module):
    """Try importing run.py, just to check if basic script passes syntax/imports checks"""
    run_py_import(module)


@pytest.mark.parametrize("module", MODULE_STEP_FOLDERS)
def test_module_spec_yaml_exists_and_is_parsable(module):
    """Try loading the module_spec.yaml file for the module"""
    module_spec_yaml_exists_and_is_parsable(module)


### ADVANCED TESTS ###
# for module implementing full design pattern (get_arg_parser())


@pytest.mark.parametrize("module", MODULE_STEP_FOLDERS)
def test_module_run_get_arg_parser(module):
    """Tests is module run.py has function get_arg_parser(parser)"""
    module_run_get_arg_parser(module)


@pytest.mark.parametrize("module", MODULE_STEP_FOLDERS)
def test_module_spec_args_against_script_argparse(module):
    """Tests alignment between module_spec arguments and script parser arguments"""
    if_arguments_from_module_spec_match_script_argparse(module)


# NOTE: this test has been disabled because it requires exception re-throw in compliant_handle()
# see pull request https://eemo.visualstudio.com/TEE/_git/TEEGit/pullrequest/16849
# @pytest.mark.parametrize("module", MODULE_MANIFEST_ADVANCED)
# def test_script_main_with_synthetic_arguments(mocker, module):
#    """Tests alignment between module_spec arguments and script parser arguments"""
#    script_main_with_synthetic_arguments(module, mocker)

### COMPLIANCE TESTS (EXPERIMENTAL) ###
# for modules running in Heron eyes-off environment


@pytest.mark.parametrize("module", MODULE_STEP_FOLDERS)
def test_module_uses_private_acr(module):
    """Tests base image in private ACR"""
    module_uses_private_acr(module, "polymerprod.azurecr.io")


@pytest.mark.parametrize("module", MODULE_STEP_FOLDERS)
def test_module_uses_private_python_feed(module):
    """Tests base image in private python feed"""
    module_uses_private_python_feed(
        module,
        "https://o365exchange.pkgs.visualstudio.com/_packaging/PolymerPythonPackages/pypi/simple/",
    )



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

import argparse
from argparse import ArgumentParser

import shrike
from shrike.compliant_logging.constants import DataCategory
from shrike.compliant_logging.exceptions import prefix_stack_trace
import logging
from pyspark.sql import SparkSession
import time


def parse_percentage(percentage) -> int:
    percentage = int(percentage)
    if percentage < 0 or percentage > 100:
        raise ArithmeticError(f"Percentage {percentage} < 0 or > 100")
    return percentage


def get_arg_parser(parser=None) -> ArgumentParser:
    rv = parser or argparse.ArgumentParser()

    rv.add_argument(
        "--input_path",
        required=True,
        type=str,
        help="Path to Heron extraction or other data to import from.",
    )

    rv.add_argument("--in_file_type", required=True, type=str, help="Input file type.")

    rv.add_argument(
        "--percent_take",
        required=True,
        type=parse_percentage,
        help="Integer percent of data to keep from this extraction.",
    )

    rv.add_argument(
        "--out_file_type", required=True, type=str, help="File type for output."
    )

    rv.add_argument(
        "--output_path",
        required=True,
        type=str,
        help="Path / filename to write the data extraction to.",
    )

    return rv


def get_spark_session(log: logging.Logger) -> SparkSession:
    start_time = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
    app_name = f"aml-ds-hdi-canary-{start_time}"
    log.info(f"application name: {app_name}", category=DataCategory.PUBLIC)

    rv = SparkSession.builder.appName(app_name).getOrCreate()
    return rv


def run(args, log):
    log.info(f"Running module with arguments {args}", category=DataCategory.PUBLIC)

    spark = get_spark_session(log)

    log.info(
        f"Reading {args.in_file_type} files from {args.input_path}",
        category=DataCategory.PUBLIC,
    )

    df = spark.read.format(args.in_file_type).load(args.input_path)
    log.info(f"Data schema: {df.columns}", category=DataCategory.PUBLIC)

    n_rows = df.count()
    log.info(f"Data has {n_rows} rows", category=DataCategory.PUBLIC)

    n_take = int(n_rows * args.percent_take / 100)
    log.info(f"Keeping {n_take} rows", category=DataCategory.PUBLIC)

    df = df.limit(n_take)

    log.info(
        f"Writing {args.out_file_type} output data to {args.output_path}",
        category=DataCategory.PUBLIC,
    )
    df.write.format(args.out_file_type).mode("overwrite").option("header", True).save(
        args.output_path
    )

    log.info("Finishing HDI probe", category=DataCategory.PUBLIC)


@prefix_stack_trace(keep_message=True, allow_list=["SystemExit"])
def main():
    parser = get_arg_parser()
    args, _ = parser.parse_known_args()

    log = logging.getLogger(__name__)

    run(args, log)


if __name__ == "__main__":
    shrike.compliant_loggine.enable_compliant_logging()
    main()



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""
Demonstrate minimal use of the Spark .NET runner functionality.
"""

from shrike.spark import run_spark_net


if __name__ == "__main__":
    run_spark_net()



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""
PyTest suite for deep testing parameters built inside a pipeline.
"""
import os
import pytest
import sys
from unittest.mock import patch

from shrike.pipeline.testing.pipeline_class_test import (
    deeptest_graph_comparison,
)

from .sample.pipeline.passthrough_test import CanaryPipelineStatsPassthrough
from .sample.pipeline.multinode_training_test import MultiNodeTrainingPipeline
from .sample.pipeline.spark_hello import SparkHelloPipeline
from .sample.pipeline.convert_tsv_to_ss_test import ConvertTsvToSSPipeline
from .sample.pipeline.federated_learning import SampleFederatedPipeline


@pytest.fixture()
def pipeline_config_path():
    """Locates the pipeline config folder for unit tests.

    Returns:
        str: path to config file in temporary folder
    """
    return os.path.join(os.path.dirname(os.path.abspath(__file__)), "sample", "conf")


def test_spark_hello_build_deep_test(pipeline_config_path):
    """Tests the graph by running the main function itself (which does .validate())"""
    # path where to generate the tested graph
    json_export_path = os.path.join(
        os.path.dirname(os.path.abspath(__file__)), "spark_hello_graph.json"
    )

    # arguments for the main function
    testargs = [
        "prog",
        "--config-dir",
        pipeline_config_path,
        "--config-name",
        "pipelines/spark_hello_test",
        "module_loader.use_local='*'",
        "+run.disable_telemetry=True",
        f"+run.export={json_export_path}",
    ]
    # will do everything except submit :)
    with patch.object(sys, "argv", testargs):
        pipeline_instance = SparkHelloPipeline.main()

    # checks if pipeline instance is actually returned
    assert (
        pipeline_instance is not None
    ), "main function should return a pipeline instance"

    # checks the exported file in temp dir
    assert os.path.isfile(json_export_path), "main function should generate a json file"

    deeptest_graph_comparison(
        # exported graph
        json_export_path,
        # reference graph
        os.path.join(
            os.path.dirname(__file__), "data", "spark_hello_export_reference.json"
        ),
    )


def test_stats_passthrough_build_deep_test(pipeline_config_path):
    """Tests the graph by running the main function itself (which does .validate())"""
    # path where to generate the tested graph
    json_export_path = os.path.join(
        os.path.dirname(os.path.abspath(__file__)), "stats_passthrough_graph.json"
    )

    # arguments for the main function
    testargs = [
        "prog",
        "--config-dir",
        pipeline_config_path,
        "--config-name",
        "pipelines/passthrough_test_uuid",
        "module_loader.use_local='*'",
        "+run.disable_telemetry=True",
        f"+run.export={json_export_path}",
    ]
    # will do everything except submit :)
    with patch.object(sys, "argv", testargs):
        pipeline_instance = CanaryPipelineStatsPassthrough.main()

    # checks if pipeline instance is actually returned
    assert (
        pipeline_instance is not None
    ), "main function should return a pipeline instance"

    deeptest_graph_comparison(
        # exported graph
        json_export_path,
        # reference graph
        os.path.join(
            os.path.dirname(__file__), "data", "stats_passthrough_export_reference.json"
        ),
    )


def test_multinode_training_build_deep_test(pipeline_config_path):
    """Tests the graph by running the main function itself (which does .validate())"""
    # path where to generate the tested graph
    json_export_path = os.path.join(
        os.path.dirname(os.path.abspath(__file__)), "multinode_training_graph.json"
    )

    # arguments for the main function
    testargs = [
        "prog",
        "--config-dir",
        pipeline_config_path,
        "--config-name",
        "pipelines/multinode_training_test",
        "module_loader.use_local='*'",
        "+run.disable_telemetry=True",
        f"+run.export={json_export_path}",
    ]
    # will do everything except submit :)
    with patch.object(sys, "argv", testargs):
        pipeline_instance = MultiNodeTrainingPipeline.main()

    # checks if pipeline instance is actually returned
    assert (
        pipeline_instance is not None
    ), "main function should return a pipeline instance"

    deeptest_graph_comparison(
        # exported graph
        json_export_path,
        # reference graph
        os.path.join(
            os.path.dirname(__file__),
            "data",
            "multinode_training_export_reference.json",
        ),
    )


def test_convert_tsv_to_ss(pipeline_config_path):
    """Tests the graph by running the main function itself (which does .validate())"""
    # path where to generate the tested graph
    json_export_path = os.path.join(
        os.path.dirname(os.path.abspath(__file__)), "convert_tsv_to_ss_test.json"
    )

    # arguments for the main function
    testargs = [
        "prog",
        "--config-dir",
        pipeline_config_path,
        "--config-name",
        "pipelines/convert_tsv_to_ss_test",
        "module_loader.use_local='*'",
        f"+run.export={json_export_path}",
    ]
    # will do everything except submit :)
    with patch.object(sys, "argv", testargs):
        pipeline_instance = ConvertTsvToSSPipeline.main()

    # checks if pipeline instance is actually returned
    assert (
        pipeline_instance is not None
    ), "main function should return a pipeline instance"

    deeptest_graph_comparison(
        # exported graph
        json_export_path,
        # reference graph
        os.path.join(
            os.path.dirname(__file__),
            "data",
            "convert_tsv_to_ss_reference.json",
        ),
    )


# def test_federated_learning(pipeline_config_path):
#     """Tests the graph by running the main function itself (which does .validate())"""
#     # path where to generate the tested graph
#     json_export_path = os.path.join(
#         os.path.dirname(os.path.abspath(__file__)), "federated_learning.json"
#     )

#     # arguments for the main function
#     testargs = [
#         "prog",
#         "--config-dir",
#         pipeline_config_path,
#         "--config-name",
#         "pipelines/federated_learning",
#         f"+run.export={json_export_path}",
#     ]
#     # will do everything except submit :)
#     with patch.object(sys, "argv", testargs):
#         pipeline_instance = SampleFederatedPipeline.main()

#     # checks if pipeline instance is actually returned
#     assert (
#         pipeline_instance is not None
#     ), "main function should return a pipeline instance"

#     deeptest_graph_comparison(
#         # exported graph
#         json_export_path,
#         # reference graph
#         os.path.join(
#             os.path.dirname(__file__),
#             "data",
#             "federated_learning_reference.json",
#         ),
#     )



"""run.py for mock components"""
import argparse
from distutils.dir_util import copy_tree


def get_arg_parser(parser=None):
    """Parse the command line arguments for merge using argparse

    Args:
        parser (argparse.ArgumentParser or CompliantArgumentParser):
        an argument parser instance

    Returns:
        ArgumentParser: the argument parser instance

    Notes:
        if parser is None, creates a new parser instance
    """
    # add arguments that are specific to the component
    if parser is None:
        parser = argparse.ArgumentParser(description=__doc__)

    for arg in ["--input_data_01", "--input_data_02", "--input_data_03", "--results"]:
        parser.add_argument(arg)
    return parser


if __name__ == "__main__":
    # get the arguments
    parser = get_arg_parser()
    args = parser.parse_args()
    args = vars(args)

    copy_tree(args["input_data_01"], args["results"])



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

    Author: Shize Su for Microsoft
    Contact: shizs@microsoft.com
"""
import os
import argparse
from subprocess import run as subprocess_run
from subprocess import PIPE, STDOUT, Popen
import subprocess
import shutil
import tempfile
import zipfile
from distutils.dir_util import copy_tree
from shrike.compliant_logging.constants import DataCategory
from shrike.compliant_logging.exceptions import prefix_stack_trace
from shrike.compliant_logging import enable_compliant_logging
import logging


def run_windows_command(one_command, log):
    try:
        log.info("command is " + one_command, category=DataCategory.PUBLIC)
        proc = subprocess_run(one_command, stdout=PIPE, stderr=STDOUT, check=True)
        log.info(proc.stdout.decode("utf-8"), category=DataCategory.PRIVATE)
    except subprocess.CalledProcessError as e:
        log.info(
            "We got error while running command. Error message is:",
            category=DataCategory.PUBLIC,
        )
        log.info(e.output.decode("utf-8"), category=DataCategory.PRIVATE)
        raise e


def get_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument("--output_dir", help="output directory", required=True)
    parser.add_argument(
        "--input_transcription_folder", help="input_transcription folder", required=True
    )
    parser.add_argument(
        "--input_transcription_filename",
        help="ex) input_text_script_audio_generate.txt",
        required=True,
    )
    parser.add_argument(
        "--input_transcription_phone_dict_filename",
        help="phone dict file name under input_transcription_folder ",
        required=True,
    )
    parser.add_argument("--InModelFold", help="input model folder ", required=True)
    parser.add_argument("--InSpeakName", type=str, default="02")
    parser.add_argument("--InTacotronStep", type=int, default=460000)

    return parser


def audio_generate_init(args, log):
    PythonEnv = tempfile.mkdtemp()
    # this path should be set in container image
    PythonEnv_zip = (
        "C:\\wkdir\\deps\\lib\\NeurallTTS_PythonEnvForTTSFeatures_compact.zip"
    )
    log.info("unzipping PythonEnv", category=DataCategory.PUBLIC)
    with zipfile.ZipFile(PythonEnv_zip, "r") as zip_ref:
        zip_ref.extractall(PythonEnv)
    log.info("PythonEnv is unzipped", category=DataCategory.PUBLIC)
    args.PythonEnv = PythonEnv

    Others_dir = tempfile.mkdtemp()
    # this path should be set in container image
    Others_zip = "C:\\wkdir\\deps\\lib\\others.zip"
    log.info("unzipping others", category=DataCategory.PUBLIC)
    with zipfile.ZipFile(Others_zip, "r") as zip_ref:
        zip_ref.extractall(Others_dir)
    log.info("others is unzipped", category=DataCategory.PUBLIC)

    log.info(
        "Copy some tools into the current working directory",
        category=DataCategory.PUBLIC,
    )

    shutil.copy(os.path.join(Others_dir, "others", "CheckPhoneSequences.exe"), ".")
    shutil.copy(os.path.join(Others_dir, "others", "GenerateSubFiles2.exe"), ".")
    copy_tree(os.path.join(Others_dir, "others", "0_Tools"), "0_Tools")


def audio_generate(args, log):
    PythonEnv = args.PythonEnv

    WorkDir = tempfile.mkdtemp()

    WorkDir1 = os.path.join(WorkDir, "1")
    OutPhoneSequencesFile = os.path.join(WorkDir, "1.1")
    log.info("Start text to phone sequence converting", category=DataCategory.PUBLIC)
    command1 = (
        "run_text_to_phone_sequences.cmd {WorkDir} {InTextScriptFile} {PythonEnv} {InSpeakName} {OutPhoneSequencesFile}"
    ).format(
        WorkDir=WorkDir1,
        InTextScriptFile=os.path.join(
            args.input_transcription_folder, args.input_transcription_filename
        ),
        PythonEnv=PythonEnv,
        InSpeakName=args.InSpeakName,
        OutPhoneSequencesFile=OutPhoneSequencesFile,
    )
    run_windows_command(command1, log)

    WorkDir2 = os.path.join(WorkDir, "2")
    OutPhoneSequencesScriptFile = os.path.join(WorkDir, "2.1")
    log.info("Check phone sequence", category=DataCategory.PUBLIC)
    command2 = (
        "run_check_phone_sequence.cmd {WorkDir} {InPhoneDictFile} {InPhoneSequencesFile} {OutPhoneSequencesScriptFile}"
    ).format(
        WorkDir=WorkDir2,
        InPhoneDictFile=os.path.join(
            args.input_transcription_folder,
            args.input_transcription_phone_dict_filename,
        ),
        InPhoneSequencesFile=OutPhoneSequencesFile,
        OutPhoneSequencesScriptFile=OutPhoneSequencesScriptFile,
    )
    run_windows_command(command2, log)

    WorkDir3 = os.path.join(WorkDir, "3")
    OutScriptFile0 = os.path.join(WorkDir, "output_script_0.txt")
    OutScriptFile1 = os.path.join(WorkDir, "output_script_1.txt")
    OutScriptFile2 = os.path.join(WorkDir, "output_script_2.txt")
    OutScriptFile3 = os.path.join(WorkDir, "output_script_3.txt")
    OutScriptFile4 = os.path.join(WorkDir, "output_script_4.txt")
    OutScriptFile5 = os.path.join(WorkDir, "output_script_5.txt")
    OutScriptFile6 = os.path.join(WorkDir, "output_script_6.txt")
    OutScriptFile7 = os.path.join(WorkDir, "output_script_7.txt")
    OutScriptFile8 = os.path.join(WorkDir, "output_script_8.txt")
    OutScriptFile9 = os.path.join(WorkDir, "output_script_9.txt")
    log.info("Generate subfiles for input", category=DataCategory.PUBLIC)
    command3 = "run_generate_sub_files_for_input.cmd {WorkDir} {InAllScriptFile} {OutScriptFile0} {OutScriptFile1} {OutScriptFile2} {OutScriptFile3} {OutScriptFile4} {OutScriptFile5} {OutScriptFile6} {OutScriptFile7} {OutScriptFile8} {OutScriptFile9}".format(
        WorkDir=WorkDir3,
        InAllScriptFile=OutPhoneSequencesScriptFile,
        OutScriptFile0=OutScriptFile0,
        OutScriptFile1=OutScriptFile1,
        OutScriptFile2=OutScriptFile2,
        OutScriptFile3=OutScriptFile3,
        OutScriptFile4=OutScriptFile4,
        OutScriptFile5=OutScriptFile5,
        OutScriptFile6=OutScriptFile6,
        OutScriptFile7=OutScriptFile7,
        OutScriptFile8=OutScriptFile8,
        OutScriptFile9=OutScriptFile9,
    )
    run_windows_command(command3, log)

    WorkDir4 = os.path.join(WorkDir, "4")
    OutMelsFile = os.path.join(WorkDir, "4.1")
    OutMelsFileAll = os.path.join(WorkDir, "4.2")
    if os.path.exists(OutMelsFileAll):
        shutil.rmtree(OutMelsFileAll)
    os.makedirs(OutMelsFileAll, exist_ok=True)
    log.info(
        "Start neural_tts_sample_generator module, text to phone sequence converting",
        category=DataCategory.PUBLIC,
    )
    command4 = (
        "run_neural_tts_sample_generator.cmd {WorkDir} {InPhoneSequencesFile} {PythonEnv} {InModelFold} {InTacotronStep} {OutMelsFile}"
    ).format(
        WorkDir=WorkDir4,
        InPhoneSequencesFile=OutScriptFile0,
        PythonEnv=PythonEnv,
        InModelFold=args.InModelFold,
        InTacotronStep=args.InTacotronStep,
        OutMelsFile=OutMelsFile,
    )
    run_windows_command(command4, log)
    shutil.copy2(
        os.path.join(OutMelsFile, "mels.txt"),
        os.path.join(OutMelsFileAll, "mels001.txt"),
    )
    shutil.copy2(
        os.path.join(OutMelsFile, "mels.scp"),
        os.path.join(OutMelsFileAll, "scps001.scp"),
    )
    files = os.listdir(OutMelsFile)
    for f in files:
        os.remove(os.path.join(OutMelsFile, f))

    command4 = (
        "run_neural_tts_sample_generator.cmd {WorkDir} {InPhoneSequencesFile} {PythonEnv} {InModelFold} {InTacotronStep} {OutMelsFile}"
    ).format(
        WorkDir=WorkDir4,
        InPhoneSequencesFile=OutScriptFile1,
        PythonEnv=PythonEnv,
        InModelFold=args.InModelFold,
        InTacotronStep=args.InTacotronStep,
        OutMelsFile=OutMelsFile,
    )
    run_windows_command(command4, log)
    shutil.copy2(
        os.path.join(OutMelsFile, "mels.txt"),
        os.path.join(OutMelsFileAll, "mels002.txt"),
    )
    shutil.copy2(
        os.path.join(OutMelsFile, "mels.scp"),
        os.path.join(OutMelsFileAll, "scps002.scp"),
    )
    files = os.listdir(OutMelsFile)
    for f in files:
        os.remove(os.path.join(OutMelsFile, f))

    command4 = (
        "run_neural_tts_sample_generator.cmd {WorkDir} {InPhoneSequencesFile} {PythonEnv} {InModelFold} {InTacotronStep} {OutMelsFile}"
    ).format(
        WorkDir=WorkDir4,
        InPhoneSequencesFile=OutScriptFile2,
        PythonEnv=PythonEnv,
        InModelFold=args.InModelFold,
        InTacotronStep=args.InTacotronStep,
        OutMelsFile=OutMelsFile,
    )
    run_windows_command(command4, log)
    shutil.copy2(
        os.path.join(OutMelsFile, "mels.txt"),
        os.path.join(OutMelsFileAll, "mels003.txt"),
    )
    shutil.copy2(
        os.path.join(OutMelsFile, "mels.scp"),
        os.path.join(OutMelsFileAll, "scps003.scp"),
    )
    files = os.listdir(OutMelsFile)
    for f in files:
        os.remove(os.path.join(OutMelsFile, f))

    command4 = (
        "run_neural_tts_sample_generator.cmd {WorkDir} {InPhoneSequencesFile} {PythonEnv} {InModelFold} {InTacotronStep} {OutMelsFile}"
    ).format(
        WorkDir=WorkDir4,
        InPhoneSequencesFile=OutScriptFile3,
        PythonEnv=PythonEnv,
        InModelFold=args.InModelFold,
        InTacotronStep=args.InTacotronStep,
        OutMelsFile=OutMelsFile,
    )
    run_windows_command(command4, log)
    shutil.copy2(
        os.path.join(OutMelsFile, "mels.txt"),
        os.path.join(OutMelsFileAll, "mels004.txt"),
    )
    shutil.copy2(
        os.path.join(OutMelsFile, "mels.scp"),
        os.path.join(OutMelsFileAll, "scps004.scp"),
    )
    files = os.listdir(OutMelsFile)
    for f in files:
        os.remove(os.path.join(OutMelsFile, f))

    command4 = (
        "run_neural_tts_sample_generator.cmd {WorkDir} {InPhoneSequencesFile} {PythonEnv} {InModelFold} {InTacotronStep} {OutMelsFile}"
    ).format(
        WorkDir=WorkDir4,
        InPhoneSequencesFile=OutScriptFile4,
        PythonEnv=PythonEnv,
        InModelFold=args.InModelFold,
        InTacotronStep=args.InTacotronStep,
        OutMelsFile=OutMelsFile,
    )
    run_windows_command(command4, log)
    shutil.copy2(
        os.path.join(OutMelsFile, "mels.txt"),
        os.path.join(OutMelsFileAll, "mels005.txt"),
    )
    shutil.copy2(
        os.path.join(OutMelsFile, "mels.scp"),
        os.path.join(OutMelsFileAll, "scps005.scp"),
    )
    files = os.listdir(OutMelsFile)
    for f in files:
        os.remove(os.path.join(OutMelsFile, f))

    command4 = (
        "run_neural_tts_sample_generator.cmd {WorkDir} {InPhoneSequencesFile} {PythonEnv} {InModelFold} {InTacotronStep} {OutMelsFile}"
    ).format(
        WorkDir=WorkDir4,
        InPhoneSequencesFile=OutScriptFile5,
        PythonEnv=PythonEnv,
        InModelFold=args.InModelFold,
        InTacotronStep=args.InTacotronStep,
        OutMelsFile=OutMelsFile,
    )
    run_windows_command(command4, log)
    shutil.copy2(
        os.path.join(OutMelsFile, "mels.txt"),
        os.path.join(OutMelsFileAll, "mels006.txt"),
    )
    shutil.copy2(
        os.path.join(OutMelsFile, "mels.scp"),
        os.path.join(OutMelsFileAll, "scps006.scp"),
    )
    files = os.listdir(OutMelsFile)
    for f in files:
        os.remove(os.path.join(OutMelsFile, f))

    command4 = (
        "run_neural_tts_sample_generator.cmd {WorkDir} {InPhoneSequencesFile} {PythonEnv} {InModelFold} {InTacotronStep} {OutMelsFile}"
    ).format(
        WorkDir=WorkDir4,
        InPhoneSequencesFile=OutScriptFile6,
        PythonEnv=PythonEnv,
        InModelFold=args.InModelFold,
        InTacotronStep=args.InTacotronStep,
        OutMelsFile=OutMelsFile,
    )
    run_windows_command(command4, log)
    shutil.copy2(
        os.path.join(OutMelsFile, "mels.txt"),
        os.path.join(OutMelsFileAll, "mels007.txt"),
    )
    shutil.copy2(
        os.path.join(OutMelsFile, "mels.scp"),
        os.path.join(OutMelsFileAll, "scps007.scp"),
    )
    files = os.listdir(OutMelsFile)
    for f in files:
        os.remove(os.path.join(OutMelsFile, f))

    command4 = (
        "run_neural_tts_sample_generator.cmd {WorkDir} {InPhoneSequencesFile} {PythonEnv} {InModelFold} {InTacotronStep} {OutMelsFile}"
    ).format(
        WorkDir=WorkDir4,
        InPhoneSequencesFile=OutScriptFile7,
        PythonEnv=PythonEnv,
        InModelFold=args.InModelFold,
        InTacotronStep=args.InTacotronStep,
        OutMelsFile=OutMelsFile,
    )
    run_windows_command(command4, log)
    shutil.copy2(
        os.path.join(OutMelsFile, "mels.txt"),
        os.path.join(OutMelsFileAll, "mels008.txt"),
    )
    shutil.copy2(
        os.path.join(OutMelsFile, "mels.scp"),
        os.path.join(OutMelsFileAll, "scps008.scp"),
    )
    files = os.listdir(OutMelsFile)
    for f in files:
        os.remove(os.path.join(OutMelsFile, f))

    command4 = (
        "run_neural_tts_sample_generator.cmd {WorkDir} {InPhoneSequencesFile} {PythonEnv} {InModelFold} {InTacotronStep} {OutMelsFile}"
    ).format(
        WorkDir=WorkDir4,
        InPhoneSequencesFile=OutScriptFile8,
        PythonEnv=PythonEnv,
        InModelFold=args.InModelFold,
        InTacotronStep=args.InTacotronStep,
        OutMelsFile=OutMelsFile,
    )
    run_windows_command(command4, log)
    shutil.copy2(
        os.path.join(OutMelsFile, "mels.txt"),
        os.path.join(OutMelsFileAll, "mels009.txt"),
    )
    shutil.copy2(
        os.path.join(OutMelsFile, "mels.scp"),
        os.path.join(OutMelsFileAll, "scps009.scp"),
    )
    files = os.listdir(OutMelsFile)
    for f in files:
        os.remove(os.path.join(OutMelsFile, f))

    command4 = (
        "run_neural_tts_sample_generator.cmd {WorkDir} {InPhoneSequencesFile} {PythonEnv} {InModelFold} {InTacotronStep} {OutMelsFile}"
    ).format(
        WorkDir=WorkDir4,
        InPhoneSequencesFile=OutScriptFile9,
        PythonEnv=PythonEnv,
        InModelFold=args.InModelFold,
        InTacotronStep=args.InTacotronStep,
        OutMelsFile=OutMelsFile,
    )
    run_windows_command(command4, log)
    shutil.copy2(
        os.path.join(OutMelsFile, "mels.txt"),
        os.path.join(OutMelsFileAll, "mels010.txt"),
    )
    shutil.copy2(
        os.path.join(OutMelsFile, "mels.scp"),
        os.path.join(OutMelsFileAll, "scps010.scp"),
    )
    files = os.listdir(OutMelsFile)
    for f in files:
        os.remove(os.path.join(OutMelsFile, f))

    # Copy output mels data to output_dir
    if os.path.exists(args.output_dir):
        shutil.rmtree(args.output_dir)
    os.makedirs(args.output_dir, exist_ok=True)
    files = os.listdir(OutMelsFileAll)
    for f in files:
        shutil.move(os.path.join(OutMelsFileAll, f), args.output_dir)

    # shutil.move(OutMelsFile, args.output_dir)
    # shutil.move(WorkDir, args.output_dir)

    shutil.rmtree(WorkDir)


if __name__ == "__main__":
    enable_compliant_logging()
    log = logging.getLogger(__name__)
    log.info("job started", category=DataCategory.PUBLIC)

    parser = get_parser()

    args, unknowns = parser.parse_known_args()
    audio_generate_init(args, log)
    audio_generate(args, log)
    log.info("job finished", category=DataCategory.PUBLIC)



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""Unit tests for checking if (subscription_id, tenant_id) pair is eyes-off"""
from shrike._core import is_eyesoff_helper
import pytest


@pytest.mark.parametrize(
    "eyes_off_subscription_id, is_eyesoff",
    [
        ("60d27411-7736-4355-ac95-ac033929fe9d", True),
        ("2ffad2ef-10c3-4793-aacc-093dc4ad2e63", False),
    ],
)
def test_is_eyesoff_Torus(eyes_off_subscription_id, is_eyesoff):
    eyes_off_tenant_id = "cdc5aeea-15c5-4db6-b079-fcadd2505dc2"

    assert is_eyesoff_helper(eyes_off_tenant_id, eyes_off_subscription_id) == is_eyesoff



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

import argparse
from pathlib import Path
import pytest
import sys
from unittest import mock


from shrike.build.core import command_line
from shrike.build.core.configuration import Configuration


class CommandForExecution(command_line.Command):
    """
    Stub subclass of `Command` for testing `execute_command`.
    """

    def __init__(self, timeout: int = 10):
        super().__init__()
        self.config = Configuration(shell_command_timeout_in_seconds=timeout)

    def run_with_config(self):
        pass


@pytest.mark.parametrize(
    "command,expected",
    [
        (["python", "--version"], f"Python {sys.version_info.major}"),
    ],
)
def test_execute_command_basic_sanity_check(command, expected, caplog):
    with caplog.at_level("INFO"):
        res = CommandForExecution().execute_command(command)

    assert expected in caplog.text
    assert res


def test_execute_command_fails_explicitly_if_attempt_run_azure_cli():
    with pytest.raises(ValueError):
        CommandForExecution().execute_command(["az", "--version"])


@pytest.mark.order(-1)
@pytest.mark.parametrize(
    "command,expected",
    [
        ("--version", "azure-cli"),
        ("extension list", "["),
        ("find --help", "I'm an AI robot"),
    ],
)
def test_execute_azure_cli_command_basic_sanity_check(command, expected, caplog):
    with caplog.at_level("INFO"):
        # Allow standard error because Azure CLI may warn that upgrades exist.
        res = CommandForExecution(20).execute_azure_cli_command(
            command, stderr_is_failure=False
        )

    assert expected in caplog.text
    assert res


@pytest.mark.parametrize(
    "command,expected",
    [
        (["pytest", "--version"], f"pytest {pytest.__version__}"),
    ],
)
def test_execute_command_stderr_and_zero_exit_code_fails(command, expected, caplog):
    res = CommandForExecution().execute_command(command)

    assert expected in caplog.text
    assert not res


@pytest.mark.parametrize(
    "command,timeout,expected",
    [
        (["pwsh", "-c", "write hi; start-sleep 10"], 8, "hi"),
    ],
)
def test_execute_command_fails_as_expected_with_timeout(
    command, timeout, expected, caplog
):
    with caplog.at_level("INFO"):
        res = CommandForExecution(timeout).execute_command(command)

    assert expected in caplog.text
    assert not res


@pytest.mark.parametrize("line", ["sample log line"])
def test_log_emphasize(caplog, line):

    with caplog.at_level("INFO"):
        with command_line._LogEmphasize(line):
            pass

    assert caplog.text.count(line) == 2


def test_command_returns_nonzero_exit_code_if_error(caplog):
    class DummyCommand(command_line.Command):
        def __init__(self):
            super().__init__()

        def run_with_config(self):
            self.register_error("error")

    with mock.patch(
        "argparse.ArgumentParser.parse_args", return_value=argparse.Namespace()
    ):
        with pytest.raises(SystemExit):
            DummyCommand().run()

    assert "errors!" in caplog.text


@pytest.mark.parametrize(
    "arm_id,sub_id,rg,workspace",
    [
        ("/_/sub/_/rg/_/_/_/ws", "sub", "rg", "ws"),
        (
            "/subscriptions/48bbc269-ce89-4f6f-9a12-c6f91fcb772d/resourceGroups/aml1p-rg/providers/Microsoft.MachineLearningServices/workspaces/aml1p-ml-wus2",
            "48bbc269-ce89-4f6f-9a12-c6f91fcb772d",
            "aml1p-rg",
            "aml1p-ml-wus2",
        ),
    ],
)
def test_parse_workspace_arm_id_works_as_expected(arm_id, sub_id, rg, workspace):
    (
        actual_sub_id,
        actual_rg,
        actual_ws,
    ) = CommandForExecution().parse_workspace_arm_id(arm_id)
    assert actual_sub_id == sub_id
    assert actual_rg == rg
    assert actual_ws == workspace


def current_drive() -> str:
    """
    Not super robust, but needed since Azure DevOps jobs run in the `D:\\` drive
    of their machines.
    """
    return __file__.split(":")[0]


@pytest.mark.parametrize(
    "path,dir,expected",
    [
        (r"\\a\\b", False, f"{current_drive()}:/a/b"),
        (r"\\a\\b", True, f"{current_drive()}:/a/b/"),
        (Path("\\a"), True, f"{current_drive()}:/a/"),
    ],
)
def test_normalize_path(path, dir, expected):
    rv = CommandForExecution().normalize_path(path, dir)

    assert expected == rv


def test_workspace_not_configured(caplog):
    with caplog.at_level("ERROR"):
        CommandForExecution().attach_workspace()
    assert "No workspaces are configured." in caplog.text



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""run.py for prs component"""
import argparse
import os
import ntpath
import time
from azureml_user.parallel_run import EntryScript
from shrike.compliant_logging.constants import DataCategory
from shrike.compliant_logging.exceptions import prefix_stack_trace
from shrike.compliant_logging import enable_compliant_logging
import logging


def get_parser():
    """Parse the command line arguments for merge using argparse
    Returns:
        ArgumentParser: the argument parser instance
    """
    # add arguments that are specific to the component
    parser = argparse.ArgumentParser()
    parser.add_argument("--output_dir", required=True)

    return parser


def init():
    global log, args
    enable_compliant_logging()
    log = logging.getLogger(__name__)
    log.info("job started. ", category=DataCategory.PUBLIC)

    parser = get_parser()
    args, _ = parser.parse_known_args()

    os.makedirs(args.output_dir, exist_ok=True)


def run(input_files):
    resultList = []

    for input_file in input_files:
        output_file = os.path.join(
            EntryScript().output_dir, ntpath.basename(input_file)
        )
        log.info("job for the input file: " + input_file, category=DataCategory.PUBLIC)
        with open(input_file, "r") as infile, open(output_file, "w") as out:
            for line in infile:
                out.write(line + " -  length of line is " + str(len(line)))

        log.info(
            "input file, " + input_file + ", was processed.",
            category=DataCategory.PUBLIC,
        )

        # Pause for a while
        time.sleep(5)
        resultList.append("dummy")

    log.info("job completed.", category=DataCategory.PUBLIC)

    return resultList



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""
PyTest suite for testing module_helper.py:

> Status: this code relates to the _recipe_ and is a _proposition_
"""
import pytest
from shrike.pipeline.module_helper import (
    AMLModuleLoader,
    module_loader_config,
    _check_use_local_syntax_valid,
)
from pathlib import Path
from omegaconf import OmegaConf


@pytest.mark.parametrize(
    "use_local,expected",
    [
        ("*", True),
        ("KEY1, KEY2", True),
        ("!KEY1, !KEY2", True),
        ("!KEY1, KEY2", False),
        ("KEY1, !KEY2", False),
    ],
)
def test_check_use_local_syntax_valid(use_local, expected):
    use_local_list = [x.strip() for x in use_local.split(",")]
    assert _check_use_local_syntax_valid(use_local_list) == expected


@pytest.mark.parametrize(
    "use_local,use_local_except_for,module_key,expected",
    [
        ("*", None, "KEY1", True),
        ("KEY1, KEY2", False, "KEY1", True),
        ("KEY1, KEY2", False, "KEY3", False),
        ("!KEY1, !KEY2", True, "KEY3", True),
        ("!KEY1, !KEY2", True, "KEY1", False),
    ],
)
def test_is_local(use_local, use_local_except_for, module_key, expected):
    # Load the testing configuration YAML file
    config = OmegaConf.load(Path(__file__).parent / "data/test_configuration.yaml")
    OmegaConf.update(config, "module_loader.use_local", use_local)

    module_loader = AMLModuleLoader(config)
    assert module_loader.use_local_except_for == use_local_except_for
    assert module_loader.is_local(module_key) == expected



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""
Sample use of Public* exception types.
"""

from shrike.compliant_logging.exceptions import prefix_stack_trace, PublicValueError


def divide(a, b):
    if not b:
        raise PublicValueError("Second argument cannot be null or zero.")
    return a / b


# Output will be:
# SystemLog: Traceback (most recent call last):
# SystemLog:   File ".\docs\logging\public-exceptions.py", line 24, in main
# SystemLog:     divide(1, 0)
# SystemLog:   File ".\docs\logging\public-exceptions.py", line 13, in divide
# SystemLog:     raise PublicValueError("Second argument cannot be null or zero.")
# SystemLog: compliant_logging.exceptions.PublicValueError: SystemLog:Second argument cannot be null or zero.
@prefix_stack_trace()
def main():
    divide(1, 0)


if __name__ == "__main__":
    main()



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

import argparse
import logging
import random
import matplotlib.pyplot as plt

from shrike.compliant_logging import enable_compliant_logging
from shrike.compliant_logging.constants import DataCategory


def run(args):
    n = args.list_length
    list1 = [random.randint(0, 100) for i in range(n)]
    list2 = [random.randint(0, 100) for i in range(n)]

    log = logging.getLogger(__name__)

    log.info(
        "Start metric logging in azure ml workspace portal",
        category=DataCategory.PUBLIC,
    )

    # log list
    log.metric_list(name="list1", value=list1, category=DataCategory.PUBLIC)

    # log table
    log.metric_table(
        name="Lists",
        value={"list1": list1, "list2": list2},
        category=DataCategory.PUBLIC,
    )

    # log scalar value
    log.metric(name="sum1", value=sum(list1), category=DataCategory.PUBLIC)
    log.metric(name="sum2", value=sum(list2), category=DataCategory.PUBLIC)

    # log image
    plt.plot(list1, list2)
    log.metric_image(name="Sample plot", plot=plt, category=DataCategory.PUBLIC)

    # log row
    for i in range(n):
        log.metric_row(
            name="pairwise-sum",
            description="",
            category=DataCategory.PUBLIC,
            pairwise_sum=list1[i] + list2[i],
        )


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--prefix", default="SystemLog:")
    parser.add_argument("--log_level", default="INFO")
    parser.add_argument(
        "--list_length",
        required=False,
        default=5,
        type=int,
        help="length of test list",
    )
    args = parser.parse_args()

    enable_compliant_logging(
        args.prefix,
        level=args.log_level,
        format="%(prefix)s%(levelname)s:%(name)s:%(message)s",
        use_aml_metrics=True,
    )

    run(args)



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.


import pytest
from shrike.spark.spark_net import *
from unittest import mock


def test_get_default_spark_session():
    mock_app_name = mock.MagicMock()
    mock_app = mock.MagicMock()

    # https://stackoverflow.com/a/34668809
    mock_app_name.getOrCreate = lambda: mock_app

    with mock.patch(
        "pyspark.sql.SparkSession.builder.appName", return_value=mock_app_name
    ):
        rv = get_default_spark_session()

    assert mock_app == rv


@pytest.mark.parametrize(
    "py_files,py_file,expected",
    [
        ("a/b/c/d.zip,a/b/e.zip", "d.zip", "a/b/c/d.zip"),
        ("a/b.zip/c.zip,a/b.zip", "b.zip", "a/b.zip"),
    ],
)
def test_full_pyfile_path(py_files, py_file, expected):
    spark = mock.MagicMock()
    spark_conf = {"spark.yarn.dist.pyFiles": py_files}

    # https://stackoverflow.com/a/39457691
    spark.sparkContext.getConf = lambda: spark_conf

    rv = full_pyfile_path(spark, py_file)
    assert expected == rv


@pytest.mark.parametrize("args", [[], [1, 2], ["a", "b", "c"]])
def test_java_args(args):
    """
    https://github.com/dkmiller/pyconfigurableml/blob/master/tests/test_entry.py
    """
    spark = mock.MagicMock()
    gateway = mock.MagicMock()
    gateway.new_array = lambda _, n: [None] * n

    with mock.patch("pyspark.SparkContext._gateway", gateway):
        rv = java_args(spark, args)

    assert rv == args


@pytest.mark.parametrize("zip,binary,args", [("foo.zip", "bar.dll", ["a", "b"])])
def test_run_spark_net_from_known_assembly(zip, binary, args):
    spark = mock.MagicMock()
    spark_conf = {"spark.yarn.dist.pyFiles": zip}

    # https://stackoverflow.com/a/39457691
    spark.sparkContext.getConf = lambda: spark_conf

    main = mock.MagicMock()
    spark._jvm.org.apache.spark.deploy.dotnet.DotnetRunner.main = main

    gateway = mock.MagicMock()
    gateway.new_array = lambda _, n: [None] * n

    with mock.patch("pyspark.SparkContext._gateway", gateway):
        run_spark_net_from_known_assembly(spark, zip, binary, args)

    # https://stackoverflow.com/a/21611963
    assert main.call_count == 1
    assert main.call_args[0][0] == [zip, binary] + args


@pytest.mark.parametrize(
    "args,expected",
    [
        (
            ["--zipFile", "foo.zip", "--binaryName", "bar.dll", "--arg1", "val1"],
            ["foo.zip", "bar.dll", "--arg1", "val1"],
        ),
        (
            ["--binaryName", "bar.dll", "--arg1", "val1", "--zipFile", "foo.zip"],
            ["foo.zip", "bar.dll", "--arg1", "val1"],
        ),
    ],
)
def test_run_spark_net(args, expected):
    gateway = mock.MagicMock()
    gateway.new_array = lambda _, n: [None] * n

    spark = mock.MagicMock()
    spark_conf = {"spark.yarn.dist.pyFiles": ",".join(args)}
    spark.sparkContext.getConf = lambda: spark_conf

    main = mock.MagicMock()
    spark._jvm.org.apache.spark.deploy.dotnet.DotnetRunner.main = main

    # https://stackoverflow.com/a/34668809
    mock_app_name = mock.MagicMock()
    mock_app_name.getOrCreate = lambda: spark

    with mock.patch("sys.argv", args):
        with mock.patch("pyspark.SparkContext._gateway", gateway):
            with mock.patch(
                "pyspark.sql.SparkSession.builder.appName", return_value=mock_app_name
            ):
                run_spark_net()

    # https://stackoverflow.com/a/21611963
    assert main.call_count == 1
    assert main.call_args[0][0] == expected



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""
PyTest suite for testing federated learning APIs.
"""
import pytest
import os
from functools import lru_cache
from omegaconf import OmegaConf
from pathlib import Path
from azureml.core.dataset import Dataset
from azureml.core import Datastore
from shrike.pipeline.federated_learning import (
    dataset_name_to_reference,
    FederatedPipelineBase,
)


@lru_cache(maxsize=None)
def federated_base():
    """
    Initiate AMLPipelineHelper class with the testing configuration.
    """
    config = OmegaConf.load(Path(__file__).parent / "data/test_configuration.yaml")
    rv = FederatedPipelineBase(config=config)
    rv.connect()
    return rv


def test_dataset_name_to_reference():
    module_instance = federated_base().component_load("stats_passthrough")
    step_instance = module_instance(input_path="foo")
    federated_base().apply_smart_runsettings(step_instance)
    outputs = dataset_name_to_reference(step_instance, ["output_path"])
    assert step_instance.outputs.output_path == outputs[0]


def test_data_transfer_without_dts_name(caplog):
    config = OmegaConf.load(Path(__file__).parent / "data/test_configuration.yaml")
    config.federated_config.data_transfer_component = ""
    fl_base = FederatedPipelineBase(config=config)
    fl_base.connect()
    with pytest.raises(Exception):
        fl_base.data_transfer([], "fake_compliant_datastore", "base_name")
    assert (
        "Please specify the data transfer component name registered in your workspace in module manifest and `federated_config.data_transfer_component` pipeline yaml."
        in caplog.text
    )


def test_data_transfer():
    fl_base = federated_base()
    stats_passthrough = fl_base.component_load("stats_passthrough")
    prev_step = stats_passthrough(input_path="foo")
    fl_base.apply_smart_runsettings(prev_step)
    base_name = fl_base.create_base_name()
    output = fl_base.data_transfer(
        [prev_step.outputs.output_path], "fake_compliant_datastore", base_name
    )[0]
    assert output.datastore == "fake_compliant_datastore"
    assert (
        output._dataset_output_options.path_on_datastore
        == base_name + "/" + "output_path"
    )


def test_process_at_orchestrator(caplog):
    fl_base = federated_base()
    with pytest.raises(Exception):
        fl_base._process_at_orchestrator("string")


def test_merge_config():
    fl_base = federated_base()
    fl_base._merge_config()
    config = fl_base.config.federated_config
    assert config.max_iterations == 2
    assert config.silos.silo1 == OmegaConf.create(
        {
            "compute": "fl-wus",
            "datastore": "fl_wus",
            "params": {"msg": "shared msg", "dataset": "test_uploaded_from_local"},
        }
    )
    assert config.silos.silo2 == OmegaConf.create(
        {
            "compute": "fl-eus",
            "datastore": "fl_eus",
            "params": {"msg": "shared msg", "dataset": "test_csv"},
        }
    )
    assert config.silos.silo3 == OmegaConf.create(
        {
            "compute": "fl-canada",
            "datastore": "fl_canada",
            "params": {"msg": "per-silo msg", "dataset": "test_csv"},
        }
    )



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""
Simple script with examples of how to directly use the function
print_prefixed_stack_trace to capture information about failed module imports.
"""

from shrike.compliant_logging.exceptions import print_prefixed_stack_trace_and_raise

try:
    # Import statement which could raise an exception containing sensitive
    # data.
    import my_custom_library  # noqa: F401
except BaseException as e:
    # Output will be:
    #
    # SystemLog: Traceback (most recent call last):
    # SystemLog:   File ".\try-except.py", line 10, in <module>
    # SystemLog:     import my_custom_library
    # SystemLog: ModuleNotFoundError: **Exception message scrubbed**
    print_prefixed_stack_trace_and_raise(err=e)

try:
    # Import statement which will never raise an exception containing sensitive
    # data.
    import another_custom_library  # noqa: F401
except BaseException as e:
    # Output will be:
    #
    # SystemLog: Traceback (most recent call last):
    # SystemLog:   File ".\try-except.py", line 17, in <module>
    # SystemLog:     import another_custom_library
    # SystemLog: ModuleNotFoundError: No module named 'another_custom_library'
    print_prefixed_stack_trace_and_raise(err=e, keep_message=True)



import argparse
import pandas as pd
import logging
from shrike.compliant_logging import enable_compliant_logging, is_eyesoff
import os


def get_arg_parser(parser=None):
    """Parse the command line arguments for merge using argparse

    Args:
        parser (argparse.ArgumentParser or CompliantArgumentParser):
        an argument parser instance

    Returns:
        ArgumentParser: the argument parser instance

    Notes:
        if parser is None, creates a new parser instance
    """
    # add arguments that are specific to the component
    if parser is None:
        parser = argparse.ArgumentParser(description=__doc__)

    parser.add_argument("--input_data", required=True, type=str, help="the input data")

    return parser


def main():
    """The main function"""

    # get the arguments
    parser = get_arg_parser()
    args = parser.parse_args()
    args = vars(args)

    # initialze logger
    enable_compliant_logging()
    logger = logging.getLogger(__name__)

    # check if workspace is eyes-off
    logger.info(f"The workspace is eyes-off: {is_eyesoff()}")

    # path of the debug dataset
    logger.info("The input dataset path is: '" + str(args["input_data"]) + "'.")

    # read the dataset
    df = pd.read_csv(args["input_data"] + "/iris.csv")

    # count the number of rows
    num_rows = df.count()[0]

    # print the result
    logger.info("The dataset contains " + str(num_rows) + " rows.")

    # get the average file size
    file_no = 0
    file_size = 0
    for path, dirs, files in os.walk(args["input_data"]):
        for f in files:
            fp = os.path.join(path, f)
            file_size += os.path.getsize(fp)
            file_no += 1
    logger.info(f"There are {file_no} files in total.")
    logger.info(f"Total file size is {file_size} bytes.")
    avg_file_size = file_size / file_no
    logger.info(f"The average file size is {avg_file_size} bytes.")


if __name__ == "__main__":
    main()



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

from a import a_method
import logging
import sys
from shrike.compliant_logging import enable_compliant_logging, DataCategory

enable_compliant_logging()
log = logging.getLogger(__name__)

if __name__ == "__main__":
    a_method()
    log.info("hi from entry point", category=DataCategory.PUBLIC)



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

import argparse
import time
import math
import os
from os import path

from azureml.core.run import Run

CHECKPOINT_FILE_NAME = "model_checkpoint.txt"


def get_logger():
    try:
        return Run.get_context()
    except Exception:
        return LocalLogger()


class LocalLogger:
    def log(self, key, value):
        print("AML-Log:", key, value)


def evaluate(num_epochs, delay_seconds, x1, x2, checkpoint_location):
    run_logger = get_logger()
    start_epoch = 0
    if checkpoint_location:
        try:
            checkpoint_file_path = os.path.join(
                checkpoint_location, CHECKPOINT_FILE_NAME
            )
            with open(checkpoint_file_path) as f:
                start_epoch = int(f.readline())
            print(
                "Found a checkpoint. Starting training from epoch: {}".format(
                    start_epoch
                )
            )
        except Exception as ex:
            print(
                "Exception occurred while trying to read checkpoint file. "
                "Starting training from epoch 0: {}".format(ex)
            )

    for ep in range(start_epoch, num_epochs):
        # x1 = 0, x2 = 0 maximize the result.
        result = math.sqrt(ep * 1000) / (1 + math.pow(x1, 2) + math.pow(x2, 2))
        result /= math.sqrt(num_epochs * 1000)  # Results between 0 and 1
        run_logger.log("result", float(result))
        time.sleep(delay_seconds)

        # Write checkpoints
        output_checkpoint_dir = "./outputs/"
        if not os.path.exists(output_checkpoint_dir):
            os.makedirs(output_checkpoint_dir)
        output_checkpoint_file_path = os.path.join(
            output_checkpoint_dir, CHECKPOINT_FILE_NAME
        )
        with open(output_checkpoint_file_path, "w") as f:
            f.write(str(ep) + "\n")


def main():
    # Get command-line arguments
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--num_epochs", type=int, default=20
    )  # Number of metrics per child run
    parser.add_argument(
        "--delay_seconds", type=float, default=20
    )  # Delay after logging a metric
    parser.add_argument("--x1", type=float, default=0, help="input x1")
    parser.add_argument("--x2", type=float, default=0, help="input x2")
    parser.add_argument(
        "--resume_from",
        type=str,
        default=None,
        help="location of the model or checkpoint files from where to resume the training",
    )

    args = parser.parse_args()

    # log parameters
    run_logger = get_logger()
    run_logger.log("args", args)

    previous_checkpoint_location = args.resume_from

    evaluate(
        args.num_epochs,
        args.delay_seconds,
        args.x1,
        args.x2,
        previous_checkpoint_location,
    )


if __name__ == "__main__":
    main()



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""
Empty script
"""
import os
import sys
import logging
import argparse


def get_arg_parser(parser=None):
    """Adds module arguments to a given argument parser (or creates one if none given).

    Args:
        parser (argparse.parser): the argument parser instance

    Returns:
        argparse.ArgumentParser: the argument parser with constructed arguments
    """
    if parser is None:
        parser = argparse.ArgumentParser(description=__doc__)

    parser.add_argument(
        "--vocab_file",
        type=str,
        help="vocab directory path (vocab generated from the train set)",
        required=True,
    )
    parser.add_argument(
        "--train_file", type=str, help="input training file path", required=True
    )
    parser.add_argument(
        "--validation_file", type=str, help="input validation file path", required=True
    )
    parser.add_argument(
        "--output_dir", type=str, help="directory to output checkpoints and metrics to"
    )

    return parser


def run(args):
    """Module run function, this does the actual work

    Args:
        args (argparse.NameSpace): arguments as specified in get_arg_parser()
    """
    pass


def main(cli_args=None):
    """Main function

    Args:
        cli_args (List[Str]): to force parsing of specific arguments (ex: from unit tests)
    """
    parser = argparse.ArgumentParser(__doc__)

    get_arg_parser(parser)

    # screams an exception and sys.exit() if there's an error
    # sends a warning on logs if there's an unknown arg
    args, _ = parser.parse_known_args(cli_args)

    run(args)


if __name__ == "__main__":
    main()



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""
This runnable graph is for a simple naive sweep component job example.
Command to submit the experiment: 
python -m pipeline.sweep_component_test --config-dir conf --config-name pipelines/sweep_component_test
"""
# pylint: disable=no-member
# NOTE: because it raises 'dict' has no 'outputs' member in dsl.pipeline construction
import os
import sys
from dataclasses import dataclass
from typing import Optional

from azure.ml.component import dsl
from shrike.pipeline.pipeline_helper import AMLPipelineHelper

# NOTE: if you need to import from pipelines.*
SMARTCOMPOSE_ROOT_PATH = os.path.abspath(
    os.path.join(os.path.dirname(__file__), "..", "..")
)
if SMARTCOMPOSE_ROOT_PATH not in sys.path:
    print(f"Adding to path: {SMARTCOMPOSE_ROOT_PATH}")
    sys.path.append(str(SMARTCOMPOSE_ROOT_PATH))


class NaiveSweepComponentPipeline(AMLPipelineHelper):
    """Runnable/reusable pipeline helper class

    This class inherits from AMLPipelineHelper which provides
    helper functions to create reusable production pipelines for SmartCompose.
    """

    @classmethod
    def get_config_class(cls):
        """Returns the config object (dataclass) for this runnable script.

        Returns:
            dataclass: class for configuring this runnable pipeline.
        """

        @dataclass
        class sweepcomponentexample:  # pylint: disable=invalid-name
            """Config object constructed as a dataclass.

            NOTE: the name of this class will be used as namespace in your config yaml file.
            See conf/reference/lstmtraining.yaml for an example.
            """

            # For the naive sweep components, it is self-contained without any explicit external dependency
            # All inputs are optional

        # return the dataclass itself
        # for helper class to construct config file
        return sweepcomponentexample

    def build(self, config):
        """Builds a pipeline function for this pipeline using AzureML SDK (dsl.pipeline).

        This method should build your graph using the provided config object.
        Your pipeline config will be under config.CONFIGNAME.*
        where CONFIGNAME is the name of the dataclass returned by get_config_class()

        This method returns a constructed pipeline function (decorated with @dsl.pipeline).

        Args:
            config (DictConfig): configuration object (see get_config_class())

        Returns:
            dsl.pipeline: the function to create your pipeline
        """
        # helper function below loads the module from registered or local version
        # depending on your config run.use_local
        sweep_component = self.component_load("sweep_component")

        # Here you should create an instance of a pipeline function (using your custom config dataclass)
        @dsl.pipeline(
            name="naive sweep component",
            description="A naive sweep component example",
            default_datastore=config.compute.compliant_datastore,
        )
        def training_pipeline_function():
            """Pipeline function for this graph.

            Args:
                naive sweep component is self-contained, and all arguments are optional

            Returns:
                dict[str->PipelineOutputData]: a dictionary of your pipeline outputs
                    for instance to be consumed by other graphs
            """
            # general syntax:
            # module_instance = module_class(input=data, param=value)
            # or
            # subgraph_instance = subgraph_function(input=data, param=value)

            sweep_component_step = sweep_component()
            # each module should be followed by this call
            # in enabled the helper class to fill-up all the settings for this module instance
            self.apply_smart_runsettings(
                sweep_component_step,
                sweep=True,
                algorithm="random",
                goal="maximize",
                slack_factor=0.1,
            )

            # return {key: output}
            return {
                "sweep_component_output_saved_model": sweep_component_step.outputs.saved_model
            }

        # finally return the function itself to be built by helper code
        return training_pipeline_function

    def pipeline_instance(self, pipeline_function, config):
        """Given a pipeline function, creates a runnable instance based on provided config.

        This is used only when calling this as a runnable pipeline using .main() function (see below).
        The goal of this function is to map the config to the pipeline_function inputs and params.

        Args:
            pipeline_function (function): the pipeline function obtained from self.build()
            config (DictConfig): configuration object (see get_config_class())

        Returns:
            azureml.core.Pipeline: the instance constructed with its inputs and params.
        """

        # when all inputs are obtained, we call the pipeline function
        experiment_pipeline = pipeline_function()

        # and we return that function so that helper can run it.
        return experiment_pipeline


# NOTE: main block is necessary only if script is intended to be run from command line
if __name__ == "__main__":
    # calling the helper .main() function
    NaiveSweepComponentPipeline.main()



# python pipelines/experiments/demo_federated_learning.py
# --config-dir pipelines/config --config-name experiments/demo_federated_learning
from shrike.pipeline import FederatedPipelineBase, StepOutput
from subgraph import DemoSubgraph
from coolname import generate_slug


class MyCoolPipeline(FederatedPipelineBase):
    @classmethod
    def required_subgraphs(cls):
        return {"DemoSubgraph": DemoSubgraph}

    def create_base_name(self) -> str:
        rv = "test-base-"
        rv += generate_slug(2)
        return rv

    def preprocess(self, config):
        preprocess_func = self.component_load("Preprocess")
        pipeline_input_dataset = self.dataset_load(
            name=config.democomponent.input_data,
            version=config.democomponent.input_data_version,
        )
        preprocess_step = preprocess_func(
            input_data=pipeline_input_dataset,
            message=config.federated_config.params.msg,
        )
        return StepOutput(preprocess_step, ["results"])

    def train(self, config, input, silo):
        # input = model weights
        train_func = self.component_load("TrainInSilo")
        input_data = self.dataset_load(silo.params.dataset)
        train_step = train_func(
            input_01=input_data, input_02=input[0], message=silo.params.msg
        )
        # step = func(input_01=input[0], message=silo.params.msg)
        return StepOutput(train_step, ["results"])

    def midprocess(self, config, input):
        demo_subgraph = self.subgraph_load("DemoSubgraph")
        midprocess_step = demo_subgraph(
            input_data_01=input[0], input_data_02=input[1], input_data_03=input[2]
        )
        return StepOutput(midprocess_step)

    def postprocess(self, config, input):
        postprocess_func = self.component_load("Postprocess")
        postprocess_step = postprocess_func(input_data=input[0])
        return StepOutput(postprocess_step, ["results"])


if __name__ == "__main__":
    MyCoolPipeline.main()



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

from pathlib import Path
import pytest
import logging

from shrike.build.utils import utils


@pytest.mark.parametrize(
    "file_name,expected_hash",
    [
        (
            "another_file_for_component1.txt",
            "9E640A18FC586A6D87716F9B4C6728B7023819E58E07C4562E0D2C14DFC3CF5B",
        ),
        (
            "spec.additional_includes",
            "50407DAA1E6DA1D91E1CE88DDDF18B3DFDA62E08B780EC9B2E8642536DD36C05",
        ),
    ],
)
def test_create_SHA_256_hash_of_file_matches_cosmic_build_tool(
    file_name, expected_hash
):
    """
    Cross-check the local "SHA 256 hash" implementation against the "Cosmic"
    build tool. These hashes were obtained from the build:

    https://dev.azure.com/msdata/Vienna/_build/results?buildId=29991141&view=artifacts&pathAsName=false&type=publishedArtifacts
    """
    file_path = str(Path(__file__).parent.parent / "steps/component1" / file_name)
    hash = utils.create_SHA_256_hash_of_file(file_path)

    assert hash == expected_hash


def test_telemetry_logger(caplog):
    """Unit tests for utils class of opencensus azure monitor"""
    telemetry_logger = utils.TelemetryLogger()
    assert (
        telemetry_logger.instrumentation_key == "aaefce9e-d109-4fac-bb9f-8277c68e91ac"
    )
    assert telemetry_logger.enable_telemetry

    telemetry_logger = utils.TelemetryLogger(enable_telemetry=False)
    assert not telemetry_logger.enable_telemetry
    with caplog.at_level("INFO"):
        telemetry_logger.log_trace(message="A unit test message. Please ignore it.")
    assert (
        "Sending trace log messages to application insight has been disabled."
        in caplog.text
    )

    telemetry_logger = utils.TelemetryLogger(enable_telemetry=True)
    telemetry_logger.log_trace(
        message="A unit test message of shrike.build. Please ignore it.",
        level=logging.INFO,
    )



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""
Demonstrate advanced use of the Spark .NET runner functionality.
"""

import argparse
from pyspark.sql import SparkSession
from shrike.spark import run_spark_net_from_known_assembly


def main(args):
    spark = SparkSession.builder.appName(args.app_name).getOrCreate()

    run_spark_net_from_known_assembly(
        spark, "dotnet-publish.zip", "assembly-name", ["--input", args.input_path]
    )


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--app-name")
    parser.add_argument("--input-path")
    args = parser.parse_args()
    main(args)



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""
PyTest suite for testing if each run.py is aligned with module specification:

> Status: this code relates to the _recipe_ and is a _proposition_
"""
import pytest
import os

from shrike.pipeline.testing.components import (
    component_spec_yaml_exists_and_is_parsable,
)
from shrike.pipeline.testing.components import component_uses_private_acr
from shrike.pipeline.testing.components import component_uses_private_python_feed
from shrike.pipeline.testing.components import component_run_py_import
from shrike.pipeline.testing.components import component_run_get_arg_parser
from shrike.pipeline.testing.components import (
    if_arguments_from_component_spec_match_script_argparse,
)

COMPONENT_ROOT_FOLDER = os.path.abspath(
    os.path.join(os.path.dirname(__file__), "sample", "steps")
)

# modules that should ALSO pass advanced tests (design pattern)
COMPONENT_FOLDERS = [
    "stats_passthrough",
    "stats_passthrough_windows",
    "multinode_trainer",
    "spark_hello",
]

COMPONENT_FILE_NAME = "module_spec.yaml"  # old style naming convention

### BASIC TESTS ###
# for basic module designs (minimal wrappers)


@pytest.mark.parametrize("component_folder", COMPONENT_FOLDERS)
def test_component_run_py_import(component_folder):
    """Try importing run.py, just to check if basic script passes syntax/imports checks"""
    component_run_py_import(
        os.path.join(COMPONENT_ROOT_FOLDER, component_folder, COMPONENT_FILE_NAME)
    )


@pytest.mark.parametrize("component_folder", COMPONENT_FOLDERS)
def test_component_spec_yaml_exists_and_is_parsable(component_folder):
    """Try loading and parsing the component spec yaml file"""
    component_spec_yaml_exists_and_is_parsable(
        os.path.join(COMPONENT_ROOT_FOLDER, component_folder, COMPONENT_FILE_NAME)
    )


### ADVANCED TESTS ###
# for module implementing full design pattern (get_arg_parser())


@pytest.mark.parametrize("component_folder", COMPONENT_FOLDERS)
def test_component_run_get_arg_parser(component_folder):
    """Tests if component run.py has function get_arg_parser(parser)"""
    component_run_get_arg_parser(
        os.path.join(COMPONENT_ROOT_FOLDER, component_folder, COMPONENT_FILE_NAME)
    )


@pytest.mark.parametrize("component_folder", COMPONENT_FOLDERS)
def test_if_arguments_from_component_spec_match_script_argparse(component_folder):
    """Tests alignment between module_spec arguments and script parser arguments"""
    if_arguments_from_component_spec_match_script_argparse(
        os.path.join(COMPONENT_ROOT_FOLDER, component_folder, COMPONENT_FILE_NAME)
    )


# NOTE: this test has been disabled because it requires exception re-throw in compliant_handle()
# see pull request https://eemo.visualstudio.com/TEE/_git/TEEGit/pullrequest/16849
# @pytest.mark.parametrize("module", MODULE_MANIFEST_ADVANCED)
# def test_script_main_with_synthetic_arguments(mocker, module):
#    """Tests alignment between module_spec arguments and script parser arguments"""
#    script_main_with_synthetic_arguments(module, mocker)

### COMPLIANCE TESTS (EXPERIMENTAL) ###
# for modules running in Heron eyes-off environment


@pytest.mark.parametrize("component_folder", COMPONENT_FOLDERS)
def test_component_uses_private_acr(component_folder):
    """Tests base image in private ACR"""
    component_uses_private_acr(
        os.path.join(COMPONENT_ROOT_FOLDER, component_folder, COMPONENT_FILE_NAME),
        "polymerprod.azurecr.io",
    )


@pytest.mark.parametrize("component_folder", COMPONENT_FOLDERS)
def test_component_uses_private_python_feed(component_folder):
    """Tests base image in private python feed"""
    component_uses_private_python_feed(
        os.path.join(COMPONENT_ROOT_FOLDER, component_folder, COMPONENT_FILE_NAME),
        "https://o365exchange.pkgs.visualstudio.com/_packaging/PolymerPythonPackages/pypi/simple/",
    )



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

import platform
import logging
from shrike._core import stream_handler

from shrike.compliant_logging import enable_compliant_logging, provide_system_info


def test_provide_system_info():
    enable_compliant_logging(prefix="SystemLog:")

    log = logging.getLogger()
    log.setLevel("INFO")

    uname = platform.uname()
    python_version = platform.python_version()

    provide_system_info(logger=log)

    with stream_handler(log, "") as context:
        provide_system_info(logger=log, library_checks=["shrike"])
        logs_1 = str(context)

    assert "### System Config ###" in logs_1
    assert f"| CPU     = {uname.processor}" in logs_1
    assert f"| Machine = {uname.machine}" in logs_1
    assert f"| System  = {uname.system} ({uname.release})" in logs_1
    assert f"| Python  = {python_version}" in logs_1
    assert "#" * 21 in logs_1
    assert "Library shrike available" in logs_1

    with stream_handler(log, "") as context:
        provide_system_info(
            logger=log, library_checks=["shrike", "SOME_UNKNOWN_LIBRARY"]
        )
        logs_2 = str(context)

    assert (
        "Library SOME_UNKNOWN_LIBRARY is not found and could not be imported" in logs_2
    )
    assert "Library shrike available" in logs_2



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""
PyTest suite for testing all runnable pipelines.
"""
import os
import pytest
import sys
import tempfile
from unittest.mock import patch

from shrike.pipeline.testing.pipeline_class_test import (
    get_config_class,
    pipeline_required_modules,
    pipeline_required_subgraphs,
)
from .sample.pipeline.passthrough_test import CanaryPipelineStatsPassthrough
from .sample.pipeline.multinode_training_test import MultiNodeTrainingPipeline
from .sample.pipeline.spark_hello import SparkHelloPipeline
from .sample.pipeline.convert_tsv_to_ss_test import ConvertTsvToSSPipeline

### Tests on the AMLPipelineHelper classes methods (unit tests)

PIPELINE_CLASSES = [
    CanaryPipelineStatsPassthrough,
    MultiNodeTrainingPipeline,
    SparkHelloPipeline,
    ConvertTsvToSSPipeline,
]


@pytest.fixture()
def pipeline_config_path():
    """Locates the pipeline config folder for unit tests.

    Returns:
        str: path to config file in temporary folder
    """
    return os.path.join(os.path.dirname(os.path.abspath(__file__)), "sample", "conf")


@pytest.mark.parametrize("pipeline_class", PIPELINE_CLASSES)
def test_pipeline_classes_get_config_class(pipeline_class):
    """Test if the get_arg_parser() method is in there and behaves correctly"""
    get_config_class(pipeline_class)


@pytest.mark.parametrize("pipeline_class", PIPELINE_CLASSES)
def test_pipeline_classes_pipeline_required_modules(pipeline_class):
    """Test if the required_modules() returns the right list of modules with all required keys"""
    pipeline_required_modules(pipeline_class)


@pytest.mark.parametrize("pipeline_class", PIPELINE_CLASSES)
def test_pipeline_classes_pipeline_required_subgraphs(pipeline_class):
    """Tests if the required_subgraphs() returns the right list of modules with all requires keys"""
    pipeline_required_subgraphs(pipeline_class)


### Tests on the validation of pipeline (integration tests)


def test_passthrough_build_local(pipeline_config_path):
    """Tests the graph by running the main function itself (which does .validate())"""
    testargs = [
        "prog",
        "--config-dir",
        pipeline_config_path,
        "--config-name",
        "pipelines/passthrough_test",
        "module_loader.use_local='*'",
    ]
    # will do everything except submit :)
    with patch.object(sys, "argv", testargs):
        CanaryPipelineStatsPassthrough.main()


def test_passthrough_uuid_build_local(pipeline_config_path):
    """Tests the graph by running the main function itself (which does .validate())"""
    testargs = [
        "prog",
        "--config-dir",
        pipeline_config_path,
        "--config-name",
        "pipelines/passthrough_test_uuid",
        "module_loader.use_local='*'",
    ]
    # will do everything except submit :)
    with patch.object(sys, "argv", testargs):
        CanaryPipelineStatsPassthrough.main()


def test_multinode_training_build_local(pipeline_config_path):
    """Tests the graph by running the main function itself (which does .validate())"""
    testargs = [
        "prog",
        "--config-dir",
        pipeline_config_path,
        "--config-name",
        "pipelines/multinode_training_test",
        "module_loader.use_local='*'",
    ]
    # will do everything except submit :)
    with patch.object(sys, "argv", testargs):
        MultiNodeTrainingPipeline.main()


def test_spark_hello_build_local(pipeline_config_path):
    """Tests the graph by running the main function itself (which does .validate())"""
    testargs = [
        "prog",
        "--config-dir",
        pipeline_config_path,
        "--config-name",
        "pipelines/spark_hello_test",
        "module_loader.use_local='*'",
    ]
    # will do everything except submit :)
    with patch.object(sys, "argv", testargs):
        SparkHelloPipeline.main()


def test_covert2ss_build_local(pipeline_config_path):
    """Tests the graph by running the main function itself (which does .validate())"""
    testargs = [
        "prog",
        "--config-dir",
        pipeline_config_path,
        "--config-name",
        "pipelines/convert_tsv_to_ss_test",
        "module_loader.use_local='*'",
    ]
    # will do everything except submit :)
    with patch.object(sys, "argv", testargs):
        ConvertTsvToSSPipeline.main()



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""Unit tests for pipeline_helper"""


from functools import lru_cache
import pytest
import sys
from omegaconf import OmegaConf
from pathlib import Path
from shrike.pipeline import AMLPipelineHelper
import yaml
import os
import shutil
from unittest.mock import MagicMock, patch


from shrike._core import stream_handler


@lru_cache(maxsize=None)
def pipeline_helper():
    """
    Initiate AMLPipelineHelper class with the testing configuration.
    """
    config = OmegaConf.load(Path(__file__).parent / "data/test_configuration.yaml")
    rv = AMLPipelineHelper(config=config)
    rv.connect()
    return rv


# Define tenant_id
tenant_id = "72f988bf-86f1-41af-91ab-2d7cd011db47"


def test_validate_experiment_name():
    """Unit tests for validate_experiment_name function"""
    with pytest.raises(ValueError):
        AMLPipelineHelper.validate_experiment_name("")
    with pytest.raises(ValueError):
        AMLPipelineHelper.validate_experiment_name("_exp-name")
    with pytest.raises(ValueError):
        AMLPipelineHelper.validate_experiment_name("wront.period")
    assert AMLPipelineHelper.validate_experiment_name("Correct-NAME_")
    assert AMLPipelineHelper.validate_experiment_name("ALLARELETTERS")
    assert AMLPipelineHelper.validate_experiment_name("12344523790")


def test_get_component_name_from_instance():
    """Unit tests for _get_component_name_from_instance"""
    component_instance = pipeline_helper().component_load(component_key="dummy_key")
    step_instance = component_instance()
    component_name = pipeline_helper()._get_component_name_from_instance(step_instance)
    assert component_name == "dummy_key"


def test_parse_pipeline_tags():
    """Unit tests for _parse_pipeline_tags"""
    assert pipeline_helper()._parse_pipeline_tags() == {"test_key": "test_value"}

    pipeline_helper().config.run.tags = '{"WRONG_JSON": 1'

    with stream_handler("shrike.pipeline.pipeline_helper", level="INFO") as handler:
        pipeline_helper()._parse_pipeline_tags()

    assert (
        'The pipeline tags {"WRONG_JSON": 1 is not a valid json-style string.'
        in str(handler)
    )

    pipeline_helper().config.run.tags = '{"test_key": "test_value"}'


@pytest.mark.parametrize(
    "windows,gpu", [(True, True), (True, False), (False, True), (False, False)]
)
def test_apply_parallel_runsettings(windows, gpu):
    """Unit tests for _apply_parallel_runsettings()"""
    # Create a module instance
    module_instance_fun = pipeline_helper().component_load(
        component_key="prscomponentlinux"
    )
    module_instance = module_instance_fun(input_dir="foo")

    if windows and gpu:
        with pytest.raises(ValueError):
            pipeline_helper()._apply_parallel_runsettings(
                module_name="prscomponentlinux",
                module_instance=module_instance,
                windows=windows,
                gpu=gpu,
            )
    else:
        with stream_handler("shrike.pipeline.pipeline_helper", level="INFO") as handler:
            pipeline_helper()._apply_parallel_runsettings(
                module_name="prscomponentlinux",
                module_instance=module_instance,
                windows=windows,
                gpu=gpu,
            )

        # Testing the stdout
        out = str(handler)
        assert "Using parallelrunstep compute target" in out
        assert f"to run {module_instance.name}" in out

        # Testing parallel runsetting parameter configuration
        assert module_instance.runsettings.parallel.error_threshold == -1
        assert module_instance.runsettings.parallel.mini_batch_size == "1"
        assert module_instance.runsettings.parallel.node_count == 10
        assert module_instance.runsettings.parallel.process_count_per_node is None
        assert module_instance.runsettings.parallel.run_invocation_timeout == 10800
        assert module_instance.runsettings.parallel.run_max_try == 3

        # Testing compute target configuration
        if windows and not gpu:
            assert module_instance.runsettings.target == "cpu-dc-win"
        elif not windows and gpu:
            assert module_instance.runsettings.target == "gpu-cluster"
        elif not windows and not gpu:
            assert module_instance.runsettings.target == "cpu-cluster"


def test_apply_scope_runsettings():
    module_instance = pipeline_helper().component_load("convert2ss")
    step_instance = module_instance(TextData="foo", ExtractionClause="foo")

    adla_account_name = "office-adhoc-c14"
    custom_job_name_suffix = "test"
    scope_param = "-tokens 50"
    pipeline_helper()._apply_scope_runsettings(
        "convert2ss",
        step_instance,
        adla_account_name=adla_account_name,
        custom_job_name_suffix=custom_job_name_suffix,
        scope_param=scope_param,
    )

    assert step_instance.runsettings.scope.adla_account_name == adla_account_name
    assert (
        step_instance.runsettings.scope.custom_job_name_suffix == custom_job_name_suffix
    )
    assert step_instance.runsettings.scope.scope_param == scope_param


def test_apply_datatransfer_runsettings():
    module_instance = pipeline_helper().component_load("data_transfer")
    step_instance = module_instance(source_data="foo")
    pipeline_helper()._apply_datatransfer_runsettings("data_Transfer", step_instance)

    assert step_instance.runsettings.target == "data-factory"


def test_apply_sweep_runsettings():
    module_instance = pipeline_helper().component_load("sweep_component")
    step_instance = module_instance()

    algorithm = "random"
    primary_metric = "result"
    goal = "maximize"
    policy_type = "median_stopping"
    evaluation_interval = 1
    delay_evaluation = 1

    # Important tip: note that we can only either specify slack_factor, or slack_amount, but not both
    # See this sweep doc for more details about sweep component: https://componentsdk.azurewebsites.net/components/sweep_component.html
    slack_factor = 0.1
    # slack_amount=0.2

    truncation_percentage = 10
    max_total_trials = 5
    max_concurrent_trials = 4
    timeout_minutes = 30

    pipeline_helper()._apply_sweep_runsettings(
        "sweep_component",
        step_instance,
        algorithm=algorithm,
        primary_metric=primary_metric,
        goal=goal,
        policy_type=policy_type,
        evaluation_interval=evaluation_interval,
        delay_evaluation=delay_evaluation,
        slack_factor=slack_factor,
        truncation_percentage=truncation_percentage,
        max_total_trials=max_total_trials,
        max_concurrent_trials=max_concurrent_trials,
        timeout_minutes=timeout_minutes,
    )

    assert step_instance.runsettings.sweep.algorithm == algorithm
    assert step_instance.runsettings.sweep.objective.primary_metric == primary_metric
    assert step_instance.runsettings.sweep.objective.goal == goal
    assert step_instance.runsettings.sweep.early_termination.policy_type == policy_type
    assert (
        step_instance.runsettings.sweep.early_termination.evaluation_interval
        == evaluation_interval
    )
    assert (
        step_instance.runsettings.sweep.early_termination.delay_evaluation
        == delay_evaluation
    )
    assert (
        step_instance.runsettings.sweep.early_termination.slack_factor == slack_factor
    )
    assert (
        step_instance.runsettings.sweep.early_termination.truncation_percentage
        == truncation_percentage
    )
    assert step_instance.runsettings.sweep.limits.max_total_trials == max_total_trials
    assert (
        step_instance.runsettings.sweep.limits.max_concurrent_trials
        == max_concurrent_trials
    )
    assert step_instance.runsettings.sweep.limits.timeout_minutes == timeout_minutes


def test2_apply_sweep_runsettings():
    module_instance = pipeline_helper().component_load("sweep_component")
    step_instance = module_instance()

    algorithm = "random"
    primary_metric = "result"
    goal = "maximize"
    policy_type = "median_stopping"
    evaluation_interval = 1
    delay_evaluation = 1

    # Note that we can only either specify slack_factor, or slack_amount, but not both
    # See this sweep doc for more details about sweep component: https://componentsdk.azurewebsites.net/components/sweep_component.html
    # slack_factor = 0.1
    slack_amount = 0.2

    truncation_percentage = 10
    max_total_trials = 5
    max_concurrent_trials = 4
    timeout_minutes = 30

    pipeline_helper()._apply_sweep_runsettings(
        "sweep_component",
        step_instance,
        algorithm=algorithm,
        primary_metric=primary_metric,
        goal=goal,
        policy_type=policy_type,
        evaluation_interval=evaluation_interval,
        delay_evaluation=delay_evaluation,
        slack_amount=slack_amount,
        truncation_percentage=truncation_percentage,
        max_total_trials=max_total_trials,
        max_concurrent_trials=max_concurrent_trials,
        timeout_minutes=timeout_minutes,
    )

    assert step_instance.runsettings.sweep.algorithm == algorithm
    assert step_instance.runsettings.sweep.objective.primary_metric == primary_metric
    assert step_instance.runsettings.sweep.objective.goal == goal
    assert step_instance.runsettings.sweep.early_termination.policy_type == policy_type
    assert (
        step_instance.runsettings.sweep.early_termination.evaluation_interval
        == evaluation_interval
    )
    assert (
        step_instance.runsettings.sweep.early_termination.delay_evaluation
        == delay_evaluation
    )
    assert (
        step_instance.runsettings.sweep.early_termination.slack_amount == slack_amount
    )
    assert (
        step_instance.runsettings.sweep.early_termination.truncation_percentage
        == truncation_percentage
    )
    assert step_instance.runsettings.sweep.limits.max_total_trials == max_total_trials
    assert (
        step_instance.runsettings.sweep.limits.max_concurrent_trials
        == max_concurrent_trials
    )
    assert step_instance.runsettings.sweep.limits.timeout_minutes == timeout_minutes


@pytest.mark.parametrize(
    "compliant,datastore",
    [(True, "fake_compliant_datastore"), (False, "workspaceblobstore")],
)
def test_apply_recommended_runsettings_datatransfer_datastore(compliant, datastore):
    module_instance = pipeline_helper().component_load("data_transfer")
    step_instance = module_instance(source_data="foo")
    pipeline_helper().apply_recommended_runsettings(
        "data_transfer", step_instance, datatransfer=True, compliant=compliant
    )

    assert step_instance.outputs.destination_data.datastore == datastore


@pytest.mark.parametrize("mpi", [True, False])
def test_apply_windows_runsettings(mpi):
    """Unit tests for _apply_windows_runsettings()"""
    # Create a module instance
    module_name = (
        "stats_passthrough_windows_mpi" if mpi else "stats_passthrough_windows"
    )
    module_instance_fun = pipeline_helper().component_load(component_key=module_name)
    module_instance = module_instance_fun(input_path="foo")

    with stream_handler("shrike.pipeline.pipeline_helper", level="INFO") as handler:
        pipeline_helper()._apply_windows_runsettings(
            module_name=module_name,
            module_instance=module_instance,
            mpi=mpi,
            node_count=2,
            process_count_per_node=3,
        )

    # Testing the stdout
    out = str(handler)
    assert (
        f"Using windows compute target cpu-dc-win to run {module_name} from pipeline class AMLPipelineHelper"
        in out
    )
    assert f"to run {module_instance.name}" in out

    # Testing mpi runsetting parameter configuration
    if mpi:
        assert module_instance.runsettings.resource_layout.node_count == 2
        assert module_instance.runsettings.resource_layout.process_count_per_node is 3

    # Testing compute target configuration
    assert module_instance.runsettings.target == "cpu-dc-win"

    # Testing input and output mode
    assert module_instance.inputs.input_path.mode == "download"
    assert module_instance.outputs.output_path.mode == "upload"


def test_apply_hdi_runsettings():
    """Unit tests for _apply_hdi_runsettings()"""
    # Create a module instance
    module_name = "SparkHelloWorld"
    module_instance_fun = pipeline_helper().component_load(component_key=module_name)
    module_instance = module_instance_fun(input_path="foo")

    with stream_handler("shrike.pipeline.pipeline_helper", level="INFO") as handler:
        pipeline_helper()._apply_hdi_runsettings(
            module_name=module_name,
            module_instance=module_instance,
            conf='{"spark.yarn.maxAppAttempts": 1, "spark.sql.shuffle.partitions": 3000}',
        )

    # Testing the stdout
    out = str(handler)
    assert (
        "Using HDI compute target cpu-cluster to run SparkHelloWorld from pipeline class AMLPipelineHelper"
        in out
    )

    # Testing HDI runsetting parameter configuration
    assert module_instance.runsettings.hdinsight.driver_memory == "2g"
    assert module_instance.runsettings.hdinsight.driver_cores == 2
    assert module_instance.runsettings.hdinsight.executor_memory == "2g"
    assert module_instance.runsettings.hdinsight.executor_cores == 2
    assert module_instance.runsettings.hdinsight.number_executors == 2
    assert (
        module_instance.runsettings.hdinsight.conf[
            "spark.yarn.appMasterEnv.DOTNET_ASSEMBLY_SEARCH_PATHS"
        ]
        == "./udfs"
    )
    assert module_instance.runsettings.hdinsight.conf["spark.yarn.maxAppAttempts"] == 1
    assert (
        module_instance.runsettings.hdinsight.conf[
            "spark.yarn.appMasterEnv.PYSPARK_PYTHON"
        ]
        == "/usr/bin/anaconda/envs/py37/bin/python3"
    )
    assert (
        module_instance.runsettings.hdinsight.conf[
            "spark.yarn.appMasterEnv.PYSPARK_DRIVER_PYTHON"
        ]
        == "/usr/bin/anaconda/envs/py37/bin/python3"
    )
    assert (
        module_instance.runsettings.hdinsight.conf["spark.sql.shuffle.partitions"]
        == 3000
    )

    # Testing compute target configuration
    assert module_instance.runsettings.target == "cpu-cluster"

    # Testing input and output mode
    assert module_instance.inputs.input_path.mode is None
    assert module_instance.outputs.output_path.mode is None


@pytest.mark.parametrize(
    "mpi,gpu", [(True, True), (True, False), (False, True), (False, False)]
)
def test_apply_linux_runsettings(mpi, gpu):
    """Unit tests for _apply_linux_runsettings()"""
    # Create a module instance
    module_name = "stats_passthrough_mpi" if mpi else "stats_passthrough"
    module_instance_fun = pipeline_helper().component_load(component_key=module_name)
    module_instance = module_instance_fun(input_path="foo")

    with stream_handler("shrike.pipeline.pipeline_helper", level="INFO") as handler:
        pipeline_helper()._apply_linux_runsettings(
            module_name=module_name,
            module_instance=module_instance,
            mpi=mpi,
            gpu=gpu,
            node_count=2 if mpi else None,
            process_count_per_node=3 if mpi else None,
        )

    out = str(handler)
    sys.stdout.write(out)

    # Testing mpi runsetting parameter configuration
    if mpi:
        assert module_instance.runsettings.resource_layout.node_count == 2
        assert module_instance.runsettings.resource_layout.process_count_per_node == 3

    # Testing compute target configuration
    if gpu:
        assert module_instance.runsettings.target == "gpu-cluster"
        assert (
            f"Using target gpu-cluster for local code GPU module {module_name} from pipeline class AMLPipelineHelper"
            in out
        )
    else:
        assert module_instance.runsettings.target == "cpu-cluster"
        assert (
            f"Using target cpu-cluster for local CPU module {module_name} from pipeline class AMLPipelineHelper"
            in out
        )

    # Testing input and output mode
    assert module_instance.inputs.input_path.mode == "download"
    assert module_instance.outputs.output_path.mode == "upload"


@pytest.mark.parametrize(
    "module_name,expected_stdout",
    [
        ("MultiNodeTrainer", "Module MultiNodeTrainer detected as MPI: True"),
        ("SparkHelloWorld", "Module SparkHelloWorld detected as HDI: True"),
        ("stats_passthrough", ""),
        (
            "stats_passthrough_windows",
            "Module stats_passthrough_windows detected as WINDOWS: True",
        ),
        (
            "stats_passthrough_windows_mpi",
            "Module stats_passthrough_windows_mpi detected as WINDOWS: True",
        ),
        (
            "stats_passthrough_windows_mpi",
            "Module stats_passthrough_windows_mpi detected as MPI: True",
        ),
        ("stats_passthrough_mpi", "Module stats_passthrough_mpi detected as MPI: True"),
        ("convert2ss", "Module convert2ss detected as SCOPE: True"),
        ("prscomponentlinux", "Module prscomponentlinux detected as PARALLEL: True"),
        ("dummy_key", ""),
        ("data_transfer", "Module data_transfer detected as DATATRANSFER: True"),
    ],
)
def test_apply_recommended_runsettings(module_name, expected_stdout):
    """Unit tests for apply_recommended_runsettings()"""
    module_instance_fun = pipeline_helper().component_load(component_key=module_name)
    if module_name == "convert2ss":
        module_instance = module_instance_fun(
            TextData="AnyFile", ExtractionClause="foo"
        )
    elif module_name == "data_transfer":
        module_instance = module_instance_fun(source_data="foo", source_type="foo")
    elif module_name == "dummy_key":
        module_instance = module_instance_fun()
    elif module_name == "MultiNodeTrainer":
        module_instance = module_instance_fun(
            vocab_file="foo", train_file="foo", validation_file="foo"
        )
    elif module_name == "prscomponentlinux":
        module_instance = module_instance_fun(input_dir="foo")
    elif module_name == "SparkHelloWorld":
        module_instance = module_instance_fun(
            input_path="foo", in_file_type="csv", percent_take=1, out_file_type="csv"
        )
    else:
        module_instance = module_instance_fun(input_path="foo")

    with stream_handler("shrike.pipeline.pipeline_helper", level="INFO") as handler:
        pipeline_helper().apply_recommended_runsettings(
            module_name=module_name,
            module_instance=module_instance,
        )

    out = str(handler)

    assert expected_stdout in out


def test_check_if_spec_yaml_override_is_needed_allow_override_false():
    pipeline_helper().config.tenant_overrides.allow_override = False
    override, _ = pipeline_helper()._check_if_spec_yaml_override_is_needed()
    assert override == False
    pipeline_helper().config.tenant_overrides.allow_override = True


def test_check_if_spec_yaml_override_is_needed_given_tenant_id():
    override, mapping = pipeline_helper()._check_if_spec_yaml_override_is_needed()
    assert override == True
    assert mapping == {
        "environment.docker.image": {
            "polymerprod.azurecr.io/training/pytorch:scpilot-rc2": "mcr.microsoft.com/azureml/base-gpu:openmpi3.1.2-cuda10.1-cudnn7-ubuntu18.04"
        },
        "tags": {"Office": "aml-ds"},
        "remove_polymer_pkg_idx": True,
    }


def test_check_if_spec_yaml_override_is_needed_given_config_filename():
    with open(Path(__file__).parent / "data/test_configuration.yaml", "r") as file:
        spec = yaml.safe_load(file)
    mapping = spec["tenant_overrides"]["mapping"][tenant_id]
    mapping["description"] = "test"
    spec["tenant_overrides"]["mapping"]["amlds"] = mapping
    spec["tenant_overrides"]["mapping"].pop(tenant_id)

    test_pipeline_helper = AMLPipelineHelper(config=OmegaConf.create(spec))
    test_pipeline_helper.connect()
    test_pipeline_helper.config.run.config_dir = os.path.join(
        Path(__file__).parent / "sample/conf"
    )

    assert "amlds" in test_pipeline_helper.config.tenant_overrides.mapping
    assert tenant_id not in test_pipeline_helper.config.tenant_overrides.mapping
    override, mapping = test_pipeline_helper._check_if_spec_yaml_override_is_needed()
    assert override == True
    assert mapping == {
        "environment.docker.image": {
            "polymerprod.azurecr.io/training/pytorch:scpilot-rc2": "mcr.microsoft.com/azureml/base-gpu:openmpi3.1.2-cuda10.1-cudnn7-ubuntu18.04"
        },
        "tags": {"Office": "aml-ds"},
        "description": "test",
        "remove_polymer_pkg_idx": True,
    }


def test_recover_spec_yaml_with_keeping_modified_files():
    file_list = [
        "tmp/spec_path.not_used",
        "tmp/spec_path.yaml",
        "tmp/env_path.not_used",
        "tmp/env_path_" + tenant_id + ".yaml",
    ]
    os.makedirs("tmp")
    for i in file_list:
        with open(i, "w") as file:
            pass
    pipeline_helper()._recover_spec_yaml([file_list], True)
    assert os.path.exists("tmp/spec_path.yaml")
    assert os.path.exists("tmp/env_path.yaml")
    assert os.path.exists("tmp/spec_path_" + tenant_id + ".yaml")
    assert os.path.exists("tmp/env_path_" + tenant_id + ".yaml")
    assert len(os.listdir("tmp")) == 4
    shutil.rmtree("tmp")


def test_recover_spec_yaml_without_keeping_modified_files():
    file_list = [
        "tmp/spec_path.not_used",
        "tmp/spec_path.yaml",
        "tmp/env_path.not_used",
        "tmp/env_path_" + tenant_id + ".yaml",
    ]
    os.makedirs("tmp")
    for i in file_list:
        with open(i, "w") as file:
            pass
    pipeline_helper()._recover_spec_yaml([file_list], False)
    assert os.path.exists("tmp/spec_path.yaml")
    assert os.path.exists("tmp/env_path.yaml")
    assert not os.path.exists("tmp/spec_path_" + tenant_id + ".yaml")
    assert not os.path.exists("tmp/env_path_" + tenant_id + ".yaml")
    assert len(os.listdir("tmp")) == 2
    shutil.rmtree("tmp")


def test_update_value_given_flattened_key():
    d = {"a": {"b": {"c": 1}}}
    pipeline_helper()._update_value_given_flattened_key(d, "a.b.c", 2)
    assert d["a"]["b"]["c"] == 2
    with pytest.raises(KeyError):
        pipeline_helper()._update_value_given_flattened_key(d, "a.b.c.d", 2)
    with pytest.raises(KeyError):
        pipeline_helper()._update_value_given_flattened_key(d, "a.c.d", 2)


def test_override_spec_yaml():
    spec_mapping = pipeline_helper().config.tenant_overrides["mapping"][tenant_id]
    yaml_to_be_recovered = pipeline_helper()._override_spec_yaml(spec_mapping)
    assert (
        len(yaml_to_be_recovered)
        == len(pipeline_helper().module_loader.modules_manifest) - 1
    )  # except for remote component "office.smart_reply.fl.data_transfer"
    for pair in yaml_to_be_recovered:
        assert len(pair) == 4
        for file_path in pair:
            if file_path:
                assert os.path.exists(file_path)
    pipeline_helper()._recover_spec_yaml(yaml_to_be_recovered, False)


def test_override_single_spec_yaml_without_environment_override():
    folder_path = os.path.join(
        Path(__file__).parent, "sample/steps/multinode_trainer_copy"
    )
    shutil.copytree(
        os.path.join(Path(__file__).parent, "sample/steps/multinode_trainer"),
        folder_path,
    )
    spec_path = os.path.join(folder_path, "module_spec.yaml")
    with open(spec_path, "r") as file:
        spec = yaml.safe_load(file)
    assert (
        spec["environment"]["docker"]["image"]
        == "polymerprod.azurecr.io/training/pytorch:scpilot-rc2"
    )
    assert not spec["tags"]["Office"]
    spec_mapping = pipeline_helper().config.tenant_overrides["mapping"][tenant_id]
    (
        old_spec_path,
        old_env_file_path,
        new_env_file_path,
    ) = pipeline_helper()._override_single_spec_yaml(spec_path, spec_mapping, False)
    assert old_spec_path == os.path.join(folder_path, "module_spec.not_used")
    assert not old_env_file_path
    assert not new_env_file_path
    with open(spec_path, "r") as file:
        spec = yaml.safe_load(file)
    assert (
        spec["environment"]["docker"]["image"]
        == "mcr.microsoft.com/azureml/base-gpu:openmpi3.1.2-cuda10.1-cudnn7-ubuntu18.04"
    )
    assert spec["tags"]["Office"] == "aml-ds"
    shutil.rmtree(folder_path)


def test_override_single_spec_yaml_with_environment_override():
    folder_path = os.path.join(
        Path(__file__).parent, "sample/steps/multinode_trainer_copy"
    )
    shutil.copytree(
        os.path.join(Path(__file__).parent, "sample/steps/multinode_trainer"),
        folder_path,
    )
    spec_path = os.path.join(folder_path, "module_spec.yaml")
    spec_mapping = pipeline_helper().config.tenant_overrides["mapping"][tenant_id]
    (
        old_spec_path,
        old_env_file_path,
        new_env_file_path,
    ) = pipeline_helper()._override_single_spec_yaml(spec_path, spec_mapping, True)
    with open(old_env_file_path, "r") as file:
        lines = file.readlines()
    assert (
        "--index-url https://o365exchange.pkgs.visualstudio.com/_packaging/PolymerPythonPackages/pypi/simple/"
        in " ".join(lines)
    )
    with open(new_env_file_path, "r") as file:
        lines = file.readlines()
    assert (
        "--index-url https://o365exchange.pkgs.visualstudio.com/_packaging/PolymerPythonPackages/pypi/simple/"
        not in " ".join(lines)
    )
    shutil.rmtree(folder_path)


def test_remove_polymer_pkg_idx_if_exists_and_save_new():
    os.makedirs("tmp")
    with open("tmp/test.txt", "w") as file:
        file.writelines(
            [
                "- --index-url https://o365exchange.pkgs.visualstudio.com/_packaging/PolymerPythonPackages/pypi/simple/",
                "foo",
            ]
        )
    (
        found_index_url,
        new_file,
        new_file_path,
        old_file_path,
    ) = pipeline_helper()._remove_polymer_pkg_idx_if_exists_and_save_new(
        "tmp",
        "test.txt",
        "--index-url https://o365exchange.pkgs.visualstudio.com/_packaging/PolymerPythonPackages/pypi/simple/",
    )
    assert found_index_url
    assert new_file == "test_" + tenant_id + ".txt"
    assert new_file_path == os.path.join("tmp", new_file)
    assert old_file_path == os.path.join("tmp", "test.not_used")
    shutil.rmtree("tmp")

    os.makedirs("tmp")
    with open("tmp/test.txt", "w"):
        pass
    (
        found_index_url,
        new_file,
        new_file_path,
        old_file_path,
    ) = pipeline_helper()._remove_polymer_pkg_idx_if_exists_and_save_new(
        "tmp",
        "test.txt",
        "--index-url https://o365exchange.pkgs.visualstudio.com/_packaging/PolymerPythonPackages/pypi/simple/",
    )
    assert not found_index_url
    assert not new_file
    assert not new_file_path
    assert not old_file_path
    shutil.rmtree("tmp")


def test_is_eyesoff():
    # will add more tests after MyData moves to Torus tenant
    assert not pipeline_helper().is_eyesoff()



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

from pathlib import Path
import pytest
import logging

from shrike.pipeline.telemetry_utils import TelemetryLogger


def test_telemetry_logger(caplog):
    """Unit tests for utils class of opencensus azure monitor"""
    telemetry_logger = TelemetryLogger()
    assert (
        telemetry_logger.instrumentation_key == "aaefce9e-d109-4fac-bb9f-8277c68e91ac"
    )
    assert telemetry_logger.enable_telemetry

    telemetry_logger = TelemetryLogger(enable_telemetry=False)
    assert not telemetry_logger.enable_telemetry
    with caplog.at_level("INFO"):
        telemetry_logger.log_trace(message="A unit test message. Please ignore it.")
    assert (
        "Sending trace log messages to application insight has been disabled."
        in caplog.text
    )

    telemetry_logger = TelemetryLogger(enable_telemetry=True)
    telemetry_logger.log_trace(
        message="A unit test message of shrike.pipeline. Please ignore it.",
        level=logging.INFO,
    )



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

import shrike.compliant_logging.stack_trace_extractor as ste
import pathlib
import re


def test_parse_trace_csharp_parses_correctly():
    """
    Verify that csharp stack traces are parsed correctly.
    """
    extractor = ste.StackTraceExtractor()

    line = (
        r"at ExtractExceptions.ExceptionExtractor.Main(String[] args "
        r"in C:\code\ExP_Code\dotnetcore\errorParser\Program.cs:line 121"
    )
    match = extractor._parse_trace_csharp(line)
    assert len(match.groupdict()) == 5

    line = (
        "Unhandled exception. System.IndexOutOfRangeException: "
        "Index was outside the bounds of the array."
    )
    match = extractor._parse_trace_csharp(line)
    assert len(match.groupdict()) == 2

    line = "hello world"
    match = extractor._parse_trace_csharp(line)
    assert not match


def test_parse_trace_python_parses_correctly():
    """
    Verify that python stack traces are parsed correctly.
    """
    extractor = ste.StackTraceExtractor()

    line = r"Traceback (most recent call last):"
    match = extractor._parse_trace_python(line)
    assert not match
    assert extractor.in_python_traceback

    line = (
        r"File \"/mnt/c/code/shrike/shrike/compliant_logging/"
        r"exceptionExtractor.py\", line 28, in <module>"
    )
    match = extractor._parse_trace_python(line)
    assert len(match.groupdict()) == 3
    assert extractor.in_python_traceback

    line = r"print(10/0)"
    match = extractor._parse_trace_python(line)
    assert not match
    assert extractor.in_python_traceback

    line = "ZeroDivisionError: division by zero"
    match = extractor._parse_trace_python(line)
    assert len(match.groupdict()) == 2
    assert not extractor.in_python_traceback

    line = "hello world"
    match = extractor._parse_trace_python(line)
    assert not match
    assert not extractor.in_python_traceback


def test_parse_file(capsys):
    """
    Verify that parsing entire file runs correctly.
    """

    HERE = pathlib.Path(__file__).parent
    file = str(HERE / "log.err")
    extractor = ste.StackTraceExtractor()
    extractor._parse_file(file)
    captured = capsys.readouterr()
    target = (
        r"^SystemLog: Parsing file .+\n"
        r"SystemLog: type: System.IndexOutOfRangeException\n"
        r"SystemLog: namespace: ExtractExceptions\n"
        r"SystemLog: class: ExceptionExtractor\n"
        r"SystemLog: method: .+\n"
        r"SystemLog: file: .+\n"
        r"SystemLog: line: 121\n\n"
        r"SystemLog: file: .+\n"
        r"SystemLog: line: 28\n"
        r"SystemLog: method: <module>\n"
        r"SystemLog: type: ZeroDivisionError\n$"
    )

    assert re.match(target, captured.out)
    assert len(captured.out.split("\n")) == 13



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""
This script is designed to convert a repo/file based on aml-ds-pipeline-contrib,
confidential-ml-utils, and aml-build-tooling to adopt the shrike package in place.
"""
import os
import re
import argparse
import logging

logger = logging.getLogger(__name__)


def check_is_target_file(file):
    if file.endswith((".py", ".yaml", ".yml", "Dockerfile")):
        return True
    elif check_is_requirements_file(file):
        return True
    else:
        return False


def check_is_requirements_file(file):
    root = os.path.split(file)[-1]
    if root.startswith("requirements-") or root == "requirements.txt":
        return True
    return False


def convert_file(keyword_pair, file):
    is_requirements_file = check_is_requirements_file(file)
    with open(file, "r", encoding="utf8") as input_file:
        lines = input_file.readlines()

    with open(file, "w", encoding="utf8") as output_file:
        for line in lines:
            if file.endswith((".py", ".yaml", ".yml")):
                for old_keyword in keyword_pair:
                    line = line.replace(old_keyword, keyword_pair[old_keyword])

            if file.endswith((".yml", ".yaml", "Dockerfile")) or is_requirements_file:
                if "Pip Authenticate aml-ds-pipeline-contrib" not in line:
                    # Replace the line of
                    # "pip install aml-ds-pipeline-contrib==0.1.9 --extra-index-url
                    # https://azuremlsdktestpypi.azureedge.net/modulesdkpreview"
                    # by "shrike[pipeline]"
                    line = re.sub(
                        "pip install aml-ds-pipeline-contrib(==|~=)([0-9\.\ \-\:\/a-zA-Z])+",  # noqa
                        "pip install shrike[pipeline]",
                        line,
                    )
                    # Replace "aml-ds-pipeline-contrib==" or "~=" by shrike
                    line = re.sub(
                        "aml-ds-pipeline-contrib(==|~=)([0-9\.a-zA-Z])+",  # noqa
                        "shrike[pipeline]",
                        line,
                    )
                    # Use case: when "aml-ds-pipeline-contrib-feed", do not replace
                    line = re.sub(
                        "(aml-ds-pipeline-contrib)([^-]|$)", "shrike\g<2>", line  # noqa
                    )
                    line = re.sub(
                        "aml-build-tooling((==|~=)([0-9\.a-zA-Z])+|$)",  # noqa
                        "shrike[build]",
                        line,
                    )
                    line = re.sub(
                        "confidential-ml-utils((==|~=)([0-9\.a-zA-Z])+|$)",  # noqa
                        "shrike",
                        line,
                    )

            output_file.write(line)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--input_path", required=True, help="Path to folder/file to be converted"
    )

    args = parser.parse_args()

    cur_repo_path = os.path.abspath(args.input_path)
    logger.info(f"Task: converting: {cur_repo_path}")

    keyword_pair = {
        "amldspipelinecontrib": "shrike.pipeline",
        "confidential_ml_utils": "shrike.compliant_logging",
        "aml_build_tooling": "shrike.build",
        "confidential_logging": "compliant_logging",
        "ConfidentialLogger": "CompliantLogger",
    }

    if os.path.isfile(cur_repo_path):
        logger.info(f"Converting {cur_repo_path}.")
        convert_file(keyword_pair, cur_repo_path)
    else:
        for root, dirs, files in os.walk(cur_repo_path):
            for file in files:
                input_path = os.path.join(root, file)
                if check_is_target_file(input_path):
                    logger.info(f"Converting {input_path}.")
                    try:
                        convert_file(keyword_pair, input_path)
                        logger.info(f"Converting {input_path} successfully.")
                    except Exception as e:
                        logger.error(
                            f"Unable to convert {input_path} with exception {e}. Please convert it manually."  # noqa
                        )


if __name__ == "__main__":
    main()



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

import logging
import sys
from shrike.compliant_logging import DataCategory


def a_method():
    log = logging.getLogger(__name__)
    log.info("hi from a", category=DataCategory.PUBLIC)



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""
Simplest example of how to use the prefix_stack_trace decorator.
"""

from shrike.compliant_logging.exceptions import prefix_stack_trace


# Output will be:
# SystemLog: Traceback (most recent call last):
# SystemLog:   File ".\hello-world.py", line 11, in main
# SystemLog:     print(1 / 0)
# SystemLog: ZeroDivisionError: **Exception message scrubbed**
@prefix_stack_trace()
def main():
    print("Hello, world!")
    print(1 / 0)


if __name__ == "__main__":
    main()



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

import io
import pickle
import pytest
import re
import sys
from traceback import TracebackException
import uuid


from shrike.compliant_logging.exceptions import (
    _PrefixStackTraceWrapper,
    prefix_stack_trace,
    SCRUB_MESSAGE,
    PREFIX,
    is_exception_allowed,
    PrefixStackTrace,
    PublicArgumentError,
    PublicRuntimeError,
    PublicValueError,
    PublicKeyError,
    PublicTypeError,
    PublicIndexError,
    PublicNotImplementedError,
    PublicFileNotFoundError,
    PublicIOError,
    print_prefixed_stack_trace_and_raise,
    scrub_exception,
)


@pytest.mark.parametrize(
    "message,exec_type",
    [("foo", ArithmeticError), ("secret data", KeyError), ("baz", Exception)],
)
def test_prefix_stack_trace_preserves_exception_type(message: str, exec_type):
    """
    Verify that the exception type and "scrub message" appear in the
    prefixed lines.
    """
    file = io.StringIO()

    @prefix_stack_trace(file)
    def function():
        raise exec_type(message)

    with pytest.raises(exec_type):
        function()

    log_lines = file.getvalue()
    assert exec_type.__name__ in log_lines
    assert SCRUB_MESSAGE in log_lines


def test_prefix_stack_trace_works_with_sys_exit():
    file = io.StringIO()

    @prefix_stack_trace(file, allow_list=["SystemExit"])
    def function():
        sys.exit(1)

    with pytest.raises(SystemExit) as e:
        function()

    assert e.value.args[0] == 1

    log_lines = file.getvalue()
    assert "SystemExit: 1" in log_lines


def test_prefix_stack_trace_works_with_file_not_found():
    file = io.StringIO()
    file_name = str(uuid.uuid4())

    @prefix_stack_trace(file, allow_list=["FileNotFoundError"])
    def function():
        with open(file_name) as f:
            str(f)

    with pytest.raises(FileNotFoundError):
        function()

    log_lines = file.getvalue()
    assert f"SystemLog: FileNotFoundError: " in log_lines
    assert file_name in log_lines


def test_prefix_stack_trace_replaces_exception_with_readonly_attribute():
    class UnmodifiableError(Exception):
        def __init__(self):
            super().__init__()

        @property
        def unmodifiable(self):
            return "you can't change me"

    file = io.StringIO()

    @prefix_stack_trace(file)
    def function():
        raise UnmodifiableError()

    with pytest.raises(PublicRuntimeError):
        function()

    log_lines = file.getvalue()
    assert (
        "SystemLog: Obtained AttributeError when trying to scrub unmodifiable from UnmodifiableError"  # noqa: E501
        in log_lines
    )


def test_prefix_stack_trace_succeeds_when_no_message():
    """
    Verify that exceptions without message are re-raised correctly.
    """
    file = io.StringIO()

    @prefix_stack_trace(file, keep_message=True)
    def function():
        assert False

    with pytest.raises(AssertionError):
        function()

    log_lines = file.getvalue()
    assert AssertionError.__name__ in log_lines


def test_prefix_stack_trace_respects_disable():
    """
    Verify that the parameter `disable` of `prefix_stack_trace` turns off the
    functionality that decorator implements.
    """
    file = io.StringIO()

    @prefix_stack_trace(file, disable=True)
    def function():
        raise Exception()

    with pytest.raises(Exception):
        function()

    assert file.getvalue() == ""


@pytest.mark.parametrize(
    "prefix,exec_type,message,keep",
    [
        ("foo", ArithmeticError, "secret data", False),
        ("baz", Exception, "you shouldn't see this", True),
    ],
)
def test_prefix_stack_trace_respects_keep_message(prefix, exec_type, message, keep):
    with pytest.raises(exec_type) as exec_info:
        with PrefixStackTrace(prefix=prefix, keep_message=keep):
            raise exec_type(message)

    assert (prefix in str(exec_info.value)) is not keep
    assert (message in str(exec_info.value)) == keep


def test_prefix_stack_trace_default_doesnt_keep_message():
    with pytest.raises(Exception) as exec_info:
        with PrefixStackTrace(prefix="pref"):
            raise Exception("msg")

    assert "msg" not in str(exec_info.value)


@pytest.mark.parametrize("prefix", ["foo__"])
def test_prefix_stack_trace_respects_prefix(prefix):
    """
    Verify that the prefix added in by `prefix_stack_trace` respects the
    provided configuration.
    """
    file = io.StringIO()

    @prefix_stack_trace(file, prefix=prefix)
    def function():
        raise Exception()

    with pytest.raises(Exception):
        function()

    assert prefix in file.getvalue()


@pytest.mark.parametrize(
    "disable,prefix,message", [(False, "pref", "mess"), (True, "foo", "bar")]
)
def test_prefix_stack_trace_respects_scrub_message(disable, prefix, message):
    """
    Verify that the "message scrubbed" string added in by `prefix_stack_trace`
    respects the provided configuration.
    """
    file = io.StringIO()

    def function():
        raise Exception(message)

    with pytest.raises(Exception):
        with PrefixStackTrace(
            disable=disable, prefix=prefix, scrub_message=message, file=file
        ):
            function()

    file_value = file.getvalue()
    if disable:
        assert "" == file_value
    else:
        assert prefix in file_value
        assert message in file_value


@pytest.mark.parametrize(
    "keep_message, allow_list, expected_result",
    [
        (
            False,
            ["arithmetic", "ModuleNotFound"],
            True,
        ),  # scrub_message with allow_list
        (False, [], False),  # scrub_message
        (True, [], True),
    ],
)  # keep_message
def test_prefix_stack_trace_nested_exception(keep_message, allow_list, expected_result):
    file = io.StringIO()

    def function1():
        import my_custom_library

        my_custom_library.foo()

    @prefix_stack_trace(file, keep_message=keep_message, allow_list=allow_list)
    def function2():
        try:
            function1()
        except ModuleNotFoundError:
            raise ArithmeticError()

    with pytest.raises(ArithmeticError):
        function2()

    assert ("No module named" in file.getvalue()) == expected_result


@pytest.mark.parametrize(
    "allow_list, expected_result",
    [
        (["ModuleNotFound"], True),  # allow_list match error type
        (["arithmetic", "ModuleNotFound"], True),  # allow_list multiple strings
        (["geometry", "algebra"], False),  # allow_list no match
        (["my_custom_library"], True),  # allow_list match error message
    ],
)
def test_prefix_stack_trace_allow_list(allow_list, expected_result):
    file = io.StringIO()
    message = "No module named"

    @prefix_stack_trace(file, allow_list=allow_list)
    def function():
        import my_custom_library

        my_custom_library.foo()

    with pytest.raises(Exception):
        function()

    assert (message in file.getvalue()) == expected_result


@pytest.mark.parametrize(
    "allow_list, expected_result",
    [
        (["argparse", "ModuleNotFound"], True),
        (["argparse", "type"], False),
        (["Bingo..+Pickle"], True),
        ([], False),
    ],
)
def test_is_exception_allowed(allow_list, expected_result):
    exception = ModuleNotFoundError("Bingo. It is a pickle.")
    res = is_exception_allowed(TracebackException.from_exception(exception), allow_list)
    assert res == expected_result


@pytest.mark.parametrize(
    "keep_message, allow_list",
    [
        (True, []),  # unscrub message
        (False, []),  # scrub message
        (False, ["ValueError"]),  # unscrub whitelisted message
    ],
)
def test_prefix_stack_trace_throws_correctly(keep_message, allow_list):
    """
    After logging the library continues execution by rethrowing an error. The final
    error thrown is picked up for error reporting by AML. It should be consistent
    with user's scrubbing choice. Verify that the scrubber preserves the exception type
    and correctly modifies the exception message
    """
    file = io.StringIO()

    message = "This is the original exception message"
    e_type = ValueError

    @prefix_stack_trace(file, keep_message=keep_message, allow_list=allow_list)
    def function():
        raise e_type(message)

    with pytest.raises(e_type) as info:
        function()

    if keep_message is True or is_exception_allowed(
        TracebackException.from_exception(info.value), allow_list
    ):
        assert message in str(info.value)
    else:
        assert SCRUB_MESSAGE in str(info.value)
    assert (PREFIX in str(info.value)) == (not keep_message and len(allow_list) == 0)
    assert info.type == e_type


@pytest.mark.parametrize(
    "allow_list,disable,file,keep_message,prefix,scrub_message, add_timestamp",
    [([], True, io.StringIO(), "keep", "prefix", "scrub", True)],
)
def test__PrefixStackTraceWrapper_is_pickleable(
    allow_list, disable, file, keep_message, prefix, scrub_message, add_timestamp
):
    pst = _PrefixStackTraceWrapper(
        file, disable, prefix, scrub_message, keep_message, allow_list, add_timestamp
    )

    data = pickle.dumps(pst)
    unpickled = pickle.loads(data)

    assert isinstance(unpickled, _PrefixStackTraceWrapper)


@pytest.mark.parametrize("add_timestamp", [(False), (True)])
def test_prefix_stack_trace_respects_add_timestamp(add_timestamp):
    """
    Verify that scrubbed stack trace includes timestamp when
    add_timestamp parameter is set to true.
    """
    file = io.StringIO()

    def function():
        raise Exception("some exception message")

    with pytest.raises(Exception):
        with PrefixStackTrace(add_timestamp=add_timestamp, file=file):
            function()

    file_value = file.getvalue()
    timestamp_regex = r" [0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2} "
    timestamp_match = re.search(timestamp_regex, file_value.split("\n")[0])
    assert bool(timestamp_match) == add_timestamp
    print("hello")


@pytest.mark.parametrize(
    "exc_type,msg",
    [
        (PublicRuntimeError, "message"),
        (PublicValueError, "message"),
        (PublicKeyError, "message"),
        (PublicTypeError, "message"),
        (PublicIndexError, "a"),
        (PublicNotImplementedError, "b"),
        (PublicFileNotFoundError, "c"),
        (PublicIOError, "d"),
    ],
)
def test_public_exception_messages_are_preserved(exc_type, msg):
    file = io.StringIO()

    with pytest.raises(exc_type):
        with PrefixStackTrace(file=file):
            raise exc_type(msg)

    file_value = file.getvalue()

    assert re.search(fr"SystemLog\:.*{msg}", file_value)


def test_public_argument_error_message_is_preserved():
    file = io.StringIO()

    with pytest.raises(PublicArgumentError):
        with PrefixStackTrace(file=file):
            raise PublicArgumentError(None, "message")

    file_value = file.getvalue()

    assert re.search(r"SystemLog\:.*message", file_value)


@pytest.mark.parametrize("message,exc_type", [("hi", ArithmeticError)])
def test_default_allow_list_respected(message, exc_type):
    from shrike.compliant_logging.exceptions import default_allow_list

    default_allow_list.clear()
    default_allow_list.append(message)

    file = io.StringIO()

    @prefix_stack_trace(file)
    def function():
        raise exc_type(message)

    with pytest.raises(exc_type):
        function()

    file_value = file.getvalue()

    assert PREFIX in file_value
    assert message in file_value


def test_print_prefixed_stack_trace_and_raise_works_with_null_exception():
    file = io.StringIO()

    with pytest.raises(Exception):
        try:
            raise Exception("boo")
        except BaseException:
            print_prefixed_stack_trace_and_raise(file)

    assert PREFIX in file.getvalue()


def test_scrub_exception_works_with_loop_in_traceback():
    """
    This test will fail if `scrub_exception` doesn't use `_seen`.
    """
    try:
        try:
            raise Exception("boo")
        except Exception as e:
            raise e from e
    except BaseException as be:
        scrubbed = scrub_exception(be, SCRUB_MESSAGE, PREFIX, True, [])

    assert scrubbed is not None


def test_scrub_exception_works_with_failed_getattr():
    """
    Test whether scrub_exception works if getattr() returns an
    error when keep_message=True
    """

    class DummyException(Exception):
        def __init__(self):
            super().__init__()

        def __getattribute__(self, name):
            if name == "args":
                raise ValueError("can't get attribute")
            elif name in dir(Exception()):
                return getattr(Exception(), name)

    with pytest.raises(ValueError):
        getattr(DummyException(), "args")

    # Expected to work
    scrub_exception(
        exception=DummyException(),
        scrub_message=SCRUB_MESSAGE,
        prefix=PREFIX,
        keep_message=True,
        allow_list=[],
    )

    # Expected to fail and raise an error
    with pytest.raises(ValueError):
        scrub_exception(
            exception=DummyException(),
            scrub_message=SCRUB_MESSAGE,
            prefix=PREFIX,
            keep_message=False,
            allow_list=[],
        )


def test_prefix_stack_trace_persist_scrubbing():
    """
    Verify that the scrubbed messages will not be re-scrubbed.
    """
    file = io.StringIO()

    @prefix_stack_trace(keep_message=True)
    def a():
        print(1 / 0)

    @prefix_stack_trace(keep_message=False, file=file)
    def b():
        print("test")
        a()

    with pytest.raises(Exception):
        b()

    file_value = file.getvalue()
    assert "ZeroDivisionError: division by zero" in file_value



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.


import numpy as np
import pandas as pd
import vaex
import platform
from packaging.version import parse
from shrike.compliant_logging.exceptions import PublicRuntimeError
import pytest

from shrike.compliant_logging.data_conversions import (
    collect_pandas_dataframe,
    collect_spark_dataframe,
    collect_vaex_dataframe,
    is_numpy_array,
    is_pandas_dataframe,
    is_pandas_series,
    is_spark_dataframe,
    is_vaex_dataframe,
    get_numpy_array_info,
    get_pandas_dataframe_info,
    get_pandas_series_info,
    get_spark_dataframe_info,
    get_vaex_dataframe_info,
    numpy_array_to_list,
    pandas_series_to_list,
    pandas_dataframe_schema,
    vaex_dataframe_schema,
    spark_dataframe_schema,
)
from shrike.compliant_logging.exceptions import PublicRuntimeError


def test_numpy_import():
    test_ndarray = np.array([1, 2, 3, 4, 5])
    assert is_numpy_array(test_ndarray)
    assert not is_numpy_array([1, 2, 3, 4, 5])
    assert "Numpy Array (Shape: (5,))" in get_numpy_array_info(test_ndarray)
    assert numpy_array_to_list(test_ndarray) == [1, 2, 3, 4, 5]


def test_pandas_import():
    test_series = pd.Series([1, 2, 3, 4, 5])

    # Create the pandas DataFrame
    test_df = pd.DataFrame(
        [["Tom", 10], ["Nick", 15], ["John", 14]], columns=["Name", "Age"]
    )

    assert is_pandas_series(test_series)
    assert not is_pandas_series([1, 2, 3, 4, 5])
    assert is_pandas_dataframe(test_df)
    assert not is_pandas_dataframe(test_series)
    assert "Pandas Series (Row Count: 5)" in get_pandas_series_info(test_series)
    assert (
        "Pandas DataFrame (Row Count: 3 / Column Count: 2)"
        in get_pandas_dataframe_info(test_df)
    )
    assert pandas_series_to_list(test_series) == [1, 2, 3, 4, 5]
    assert pandas_dataframe_schema(test_df) == {"Name": "object", "Age": "int64"}
    assert collect_pandas_dataframe(test_df) == {
        "Name": ["Tom", "Nick", "John"],
        "Age": [10, 15, 14],
    }


def test_vaex_import():
    test_df = vaex.from_arrays(x=np.arange(5), y=np.arange(5) ** 2)
    assert is_vaex_dataframe(test_df)
    assert "Vaex DataFrame (Row Count: 5 / Column Count: 2)" in get_vaex_dataframe_info(
        test_df
    )
    assert "x" in vaex_dataframe_schema(test_df)
    assert "y" in vaex_dataframe_schema(test_df, {"x": "x", "y": "y"})
    assert collect_vaex_dataframe(test_df) == {
        "x": [0, 1, 2, 3, 4],
        "y": [0, 1, 4, 9, 16],
    }
    assert collect_vaex_dataframe([1, 2, 3, 4, 5]) is None


def test_pyspark_import():
    from pyspark.sql import SparkSession
    from pyspark.sql.types import StructType, StructField, StringType, IntegerType

    test_data = [
        ("James", "", "Smith", "36636", "M", 3000),
        ("Michael", "Rose", "", "40288", "M", 4000),
        ("Robert", "", "Williams", "42114", "M", 4000),
        ("Maria", "Anne", "Jones", "39192", "F", 4000),
        ("Jen", "Mary", "Brown", "", "F", -1),
    ]

    test_schema = StructType(
        [
            StructField("firstname", StringType(), True),
            StructField("middlename", StringType(), True),
            StructField("lastname", StringType(), True),
            StructField("id", StringType(), True),
            StructField("gender", StringType(), True),
            StructField("salary", IntegerType(), True),
        ]
    )
    test_df = (
        SparkSession.builder.appName("SparkUnitTests")
        .getOrCreate()
        .createDataFrame(data=test_data, schema=test_schema)
    )

    assert is_spark_dataframe(test_df)
    assert not is_spark_dataframe([1, 2, 3, 4, 5])

    # get_spark_dataframe_info() may fail if JAVA version is not expected
    get_spark_dataframe_info(test_df)

    assert spark_dataframe_schema(test_df) == {
        "firstname": "string",
        "middlename": "string",
        "lastname": "string",
        "id": "string",
        "gender": "string",
        "salary": "int",
    }

    collect_spark_dataframe(test_df)



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    This is a test AML Module for SmartCompose.
    This copies inputs to outputs and displays some stats.

    Author: Jeff Omhover for Microsoft
    Contact: Jeff.Omhover@microsoft.com
"""
import os
import sys
import argparse
import logging
import traceback
from distutils import dir_util


def get_arg_parser(parser=None):
    """Adds module arguments to a given argument parser (or creates one if none given).

    Args:
        parser (argparse.parser): the argument parser instance

    Returns:
        argparse.ArgumentParser: the argument parser with constructed arguments
    """
    if parser is None:
        parser = argparse.ArgumentParser(description=__doc__)

    # those are examples
    group = parser.add_argument_group("Module arguments [specific]")
    group.add_argument(
        "--input_path",
        dest="input_path",
        default=None,
        type=str,
        required=True,
        help="path to input data",
    )
    group.add_argument(
        "--output_path",
        dest="output_path",
        default=None,
        type=str,
        required=True,
        help="path to output data",
    )

    return parser


def run(args):
    """Module run function, this does the actual work

    Args:
        args (argparse.NameSpace): arguments as specified in get_arg_parser()
    """
    # at the end of your run, this utility function will count files and size in the provided path
    # if using --log-in-aml=True inside an AML experiment this will send this in AML
    # IMPORTANT: please make sure you use this in your module
    os.makedirs(args.output_path, exist_ok=True)
    dir_util.copy_tree(args.input_path, args.output_path)


def main(cli_args=None):
    """Main function

    Args:
        cli_args (List[Str]): to force parsing of specific arguments (ex: from unit tests)
    """
    parser = argparse.ArgumentParser(__doc__)

    get_arg_parser(parser)

    # screams an exception and sys.exit() if there's an error
    # sends a warning on logs if there's an unknown arg
    args, _ = parser.parse_known_args(cli_args)

    run(args)


if __name__ == "__main__":
    main()



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""
Simple script with examples of how to use "with statements" to capture
information about failed module imports.
"""

from shrike.compliant_logging.exceptions import PrefixStackTrace

# Output will be:
#
# SystemLog: Traceback (most recent call last):
# SystemLog:   File ".\docs\logging\with-statement.py", line 11, in <module>
# SystemLog:     import my_custom_library  # noqa: F401
# SystemLog: ModuleNotFoundError: **Exception message scrubbed**

with PrefixStackTrace():
    # Import statement which could raise an exception containing sensitive
    # data.
    import my_custom_library  # noqa: F401

# Output will be:
#
# SystemLog: Traceback (most recent call last):
# SystemLog:   File ".\docs\logging\with-statement.py", line 22, in <module>
# SystemLog:     import another_custom_library  # noqa: F401
# SystemLog: ModuleNotFoundError: No module named 'another_custom_library'
with PrefixStackTrace(keep_message=True):
    # Import statement which will never raise an exception containing sensitive
    # data.
    import another_custom_library  # noqa: F401



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""
Constructor for a test pipeline for SmartCompose.
"""
import os
import sys
import argparse

from azure.ml.component import dsl
from azureml.core import Dataset
from shrike.pipeline.pipeline_helper import AMLPipelineHelper

from dataclasses import dataclass
from typing import Dict
from omegaconf import MISSING, OmegaConf

# NOTE: if you need to import from pipelines.*
SAMPLE_ROOT_PATH = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
if SAMPLE_ROOT_PATH not in sys.path:
    print(f"Adding to path: {SAMPLE_ROOT_PATH}")
    sys.path.append(str(SAMPLE_ROOT_PATH))

from subgraph.build_time_conditional_passthrough_subgraph import (
    BuidTimeConditionalPassthroughSubgraph,
)
from subgraph.passthrough_subgraph import PassthroughSubgraph


class CanaryPipelineStatsPassthrough(AMLPipelineHelper):
    """sample pipeline builder class"""

    @classmethod
    def get_config_class(cls):
        @dataclass
        class passthrough:  # pylint: disable=invalid-name
            input_dataset: str = "dummy_dataset"

        return passthrough

    @classmethod
    def required_subgraphs(cls):
        return {
            "PassthroughSubgraph": PassthroughSubgraph,
            "BuidTimeConditionalPassthrough": BuidTimeConditionalPassthroughSubgraph,
        }

    def build(self, config):
        """Builds a pipeline function for this pipeline.

        Args:
            config (DictConfig): configuration object (see get_config_class())

        Returns:
            pipeline_function: the function to create your pipeline
        """
        # load subgrqaph without custom paramters outside pipeline definition
        passthrough_func = self.subgraph_load("PassthroughSubgraph")
        # create an instance of a pipeline function using parameters
        @dsl.pipeline(
            name="TutorialPipeline",
            description="Just runs the Probe module in AML",
            default_datastore=config.compute.compliant_datastore,
        )
        def tutorial_pipeline_function(input_dataset):
            """Constructs a sequence of steps to ...

            Args:
                None

            Returns:
                dict: output_path
            """

            passthrough_step = passthrough_func(input_dataset)

            # load subgrqaph without custom paramters inside pipeline definition
            conditional_passthrough_1_func = self.subgraph_load(
                "BuidTimeConditionalPassthrough",
                custom_config=OmegaConf.create(
                    {"subgraph": {"passthrough_windows": True}}
                ),
            )
            conditional_passthrough_1_step = conditional_passthrough_1_func(
                input_dataset
            )

            conditional_passthrough_2_func = self.subgraph_load(
                "BuidTimeConditionalPassthrough",
                custom_config=OmegaConf.create(
                    {"subgraph": {"passthrough_windows": False}}
                ),
            )
            conditional_passthrough_2_step = conditional_passthrough_2_func(
                input_dataset
            )

            return {"output_path": conditional_passthrough_1_step.outputs.output_path}

        return tutorial_pipeline_function

    def pipeline_instance(self, pipeline_function, config):
        """Creates an instance of the pipeline using arguments.

        Args:
            pipeline_function (function): the pipeline function obtained from self.build()
            config (DictConfig): configuration object (see get_config_class())

        Returns:
            pipeline: the instance constructed using build() function
        """
        pipeline_input_dataset = self.dataset_load(config.passthrough.input_dataset)

        runnable_pipeline = pipeline_function(pipeline_input_dataset)
        return runnable_pipeline


if __name__ == "__main__":
    CanaryPipelineStatsPassthrough.main()



import argparse


def get_arg_parser(parser=None):
    """Parse the command line arguments for merge using argparse

    Args:
        parser (argparse.ArgumentParser or CompliantArgumentParser):
        an argument parser instance

    Returns:
        ArgumentParser: the argument parser instance

    Notes:
        if parser is None, creates a new parser instance
    """
    # add arguments that are specific to the component
    if parser is None:
        parser = argparse.ArgumentParser(description=__doc__)

    parser.add_argument(
        "--value",
        required=False,
        type=int,
        default=100,
        help="the value on which we will operate",
    )
    return parser


def main():
    """The main function"""

    # get the arguments
    parser = get_arg_parser()
    args = parser.parse_args()
    args = vars(args)

    # this shows a basic operation on the value passed as parameter
    value = args["value"]
    operand = 1000
    result = value + operand
    print(
        "The value passed as parameter is: "
        + str(value)
        + ". We computed: "
        + str(value)
        + " + "
        + str(operand)
        + " = "
        + str(result)
        + "."
    )


if __name__ == "__main__":
    main()
